@@ -5,14 +5,14 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +47,7 @@
 #include <getopt.h>
 
 #include <pci_types.h>
+#include <lock.h>
 #include <accessClient_types.h>
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
@@ -54,73 +55,138 @@
 #define str(x) #x
 
 #define CHECK_ERROR(func, msg)  \
-    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
+    if ((func) < 0) { \
+        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
+    }
 
 #define CHECK_FILE_ERROR(func, msg)  \
-    if ((func) == 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
+    if ((func) == 0) { \
+        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
+    }
 
 
 #define EXIT_IF_ERROR(func, msg)  \
-    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); stop_daemon(); exit(EXIT_FAILURE); }
+    if ((func) < 0) { \
+        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
+        stop_daemon(); \
+        exit(EXIT_FAILURE); \
+    }
 
 
-#define CPUID                    \
-    __asm__ volatile ("cpuid"    \
-            : "=a" (eax),            \
-            "=b" (ebx)             \
+#define CPUID \
+    __asm__ volatile ("cpuid" \
+            : "=a" (eax), "=b" (ebx) \
             : "0" (eax))
 
 
-#define  P6_FAMILY        0x6U
-#define  K8_FAMILY        0xFU
-#define  K10_FAMILY       0x10U
-#define  K15_FAMILY       0x15U
-
+/* Intel P6 */
+#define PENTIUM_M_BANIAS     0x09U
+#define PENTIUM_M_DOTHAN     0x0DU
+#define CORE_DUO             0x0EU
+#define CORE2_65             0x0FU
+#define CORE2_45             0x17U
+#define ATOM                 0x1CU
+#define ATOM_45              0x26U
+#define ATOM_32              0x36U
+#define ATOM_22              0x27U
+#define ATOM_SILVERMONT      0x4DU
+#define NEHALEM              0x1AU
+#define NEHALEM_BLOOMFIELD   0x1AU
+#define NEHALEM_LYNNFIELD    0x1EU
+#define NEHALEM_LYNNFIELD_M  0x1FU
+#define NEHALEM_WESTMERE     0x2CU
+#define NEHALEM_WESTMERE_M   0x25U
 #define SANDYBRIDGE          0x2AU
 #define SANDYBRIDGE_EP       0x2DU
+#define HASWELL              0x3CU
+#define HASWELL_EX           0x3FU
+#define HASWELL_M1           0x45U
+#define HASWELL_M2           0x46U
+#define IVYBRIDGE            0x3AU
+#define IVYBRIDGE_EP         0x3EU
+#define NEHALEM_EX           0x2EU
+#define WESTMERE_EX          0x2FU
+#define XEON_MP              0x1DU
+
+/* Intel MIC */
+#define XEON_PHI           0x01U
+
+/* AMD K10 */
+#define BARCELONA      0x02U
+#define SHANGHAI       0x04U
+#define ISTANBUL       0x08U
+#define MAGNYCOURS     0x09U
+
+/* AMD K8 */
+#define OPTERON_SC_1MB  0x05U
+#define OPTERON_DC_E    0x21U
+#define OPTERON_DC_F    0x41U
+#define ATHLON64_X2     0x43U
+#define ATHLON64_X2_F   0x4BU
+#define ATHLON64_F1     0x4FU
+#define ATHLON64_F2     0x5FU
+#define ATHLON64_X2_G   0x6BU
+#define ATHLON64_G1     0x6FU
+#define ATHLON64_G2     0x7FU
+
+
+#define  P6_FAMILY        0x6U
+#define  MIC_FAMILY       0xBU
+#define  NETBURST_FAMILY  0xFFU
+#define  K15_FAMILY       0x15U
+#define  K16_FAMILY       0x16U
+#define  K10_FAMILY       0x10U
+#define  K8_FAMILY        0xFU
 
 #define PCI_ROOT_PATH    "/proc/bus/pci/"
 #define MAX_PATH_LENGTH   60
 #define MAX_NUM_NODES    4
 
+/* Lock file controlled from outside which prevents likwid to start.
+ * Can be used to synchronize access to the hardware counters
+ * with an external monitoring system. */
+
 /* #####   TYPE DEFINITIONS   ########### */
 typedef int (*FuncPrototype)(uint32_t);
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 static int sockfd = -1;
-static int CONNFD[MAX_NUM_THREADS];
 static int connfd = -1; /* temporary in to make it compile */
-static pid_t lockID=0;
 static char* filepath;
 static const char* ident = "accessD";
 static FuncPrototype allowed = NULL;
-static int sysdaemonmode = 0;
 static int FD_MSR[MAX_NUM_THREADS];
 static int FD_PCI[MAX_NUM_NODES][MAX_NUM_DEVICES];
-static int isSandyBridge = 0;
+static int isPCIUncore = 0;
 
 static char* pci_DevicePath[MAX_NUM_DEVICES] = {
-    "13.5", "13.6", "13.1", "10.0", "10.1", "10.4",
-    "10.5", "0e.1", "08.0", "09.0", "08.6", "09.6",
-    "08.0", "09.0" };
+ "13.5",   /* PCI_R3QPI_DEVICE_LINK_0 */
+ "13.6",   /* PCI_R3QPI_DEVICE_LINK_1 */
+ "13.1",   /* PCI_R2PCIE_DEVICE */
+ "10.0",   /* PCI_IMC_DEVICE_CH_0 */
+ "10.1",   /* PCI_IMC_DEVICE_CH_1 */
+ "10.4",   /* PCI_IMC_DEVICE_CH_2 */
+ "10.5",   /* PCI_IMC_DEVICE_CH_3 */
+ "0e.1",   /* PCI_HA_DEVICE */
+ "08.2",   /* PCI_QPI_DEVICE_PORT_0 */
+ "09.2",   /* PCI_QPI_DEVICE_PORT_1 */
+ "08.6",   /* PCI_QPI_MASK_DEVICE_PORT_0 */
+ "09.6",   /* PCI_QPI_MASK_DEVICE_PORT_1 */
+ "08.0",   /* PCI_QPI_MISC_DEVICE_PORT_0 */
+ "09.0" }; /* PCI_QPI_MISC_DEVICE_PORT_1 */
+
 static char pci_filepath[MAX_PATH_LENGTH];
 
-/* Socket to bus mapping
+/* Socket to bus mapping -- will be determined at runtime;
+ * typical mappings are:
  * Socket  Bus (2S)  Bus (4s)
  *   0        0xff      0x3f
  *   1        0x7f      0x7f
  *   2                  0xbf
  *   3                  0xff
  */
+static char* socket_bus[MAX_NUM_NODES];
 
-static char* socket_bus[] = { "7f/", "ff/",  "bf/",  "ff/" };
-
-#ifndef ALLOWSYSDAEMONMODE
-#define ALLOWSYSDAEMONMODE 1
-#endif
-#ifndef SYSDAEMONSOCKETPATH
-#define SYSDAEMONSOCKETPATH "/var/run/likwid-msrd.sock"
-#endif
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
@@ -129,8 +195,13 @@ static int allowed_intel(uint32_t reg)
     if ( ((reg & 0x0F8U) == 0x0C0U) ||
             ((reg & 0xFF0U) == 0x180U) ||
             ((reg & 0xF00U) == 0x300U) ||
+            ((reg & 0xF00U) == 0xC00U) ||
+            ((reg & 0xF00U) == 0xD00U) ||
+            ((reg & 0xF00U) == 0xE00U) ||
             (reg == 0x1A0)  ||
             (reg == 0x0CE)  ||
+            (reg == 0x19C)  ||
+            (reg == 0x1A2)  ||
             (reg == 0x1AD)  ||
             (reg == 0x1A6))
     {
@@ -142,6 +213,44 @@ static int allowed_intel(uint32_t reg)
     }
 }
 
+static int allowed_silvermont(uint32_t reg)
+{
+    if ( ((reg & 0x0F8U) == 0x0C0U) ||
+            ((reg & 0xFF0U) == 0x180U) ||
+            ((reg & 0xF00U) == 0x300U) ||
+            ((reg & 0xF00U) == 0x600U) ||
+            ((reg & 0xF00U) == 0xC00U) ||
+            ((reg & 0xF00U) == 0xD00U) ||
+            (reg == 0x1A0)  ||
+            (reg == 0x0CE)  ||
+            (reg == 0x1AD)  ||
+            (reg == 0x19C)  ||
+            (reg == 0x1A2)  ||
+            (reg == 0x1A6) ||
+            (reg == 0x1A6) ||
+            (reg == 0x1A7))
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;
+    }
+}
+
+static int allowed_westmereEX(uint32_t reg)
+{
+    if (allowed_intel(reg) == 1)
+    {
+        return 1;
+    }
+    else if ((reg & 0xF00) == 0xF00)
+    {
+        return 1;
+    }
+    return 0;
+}
+
 static int allowed_sandybridge(uint32_t reg)
 {
     if ( ((reg & 0x0F8U) == 0x0C0U) ||
@@ -153,6 +262,8 @@ static int allowed_sandybridge(uint32_t reg)
             (reg == 0x1A0)  ||
             (reg == 0x0CE)  ||
             (reg == 0x1AD)  ||
+            (reg == 0x19C)  ||
+            (reg == 0x1A2)  ||
             (reg == 0x1A6))
     {
         return 1;
@@ -163,6 +274,30 @@ static int allowed_sandybridge(uint32_t reg)
     }
 }
 
+static int allowed_haswell(uint32_t reg)
+{
+    if ( ((reg & 0x0F8U) == 0x0C0U) ||
+            ((reg & 0xFF0U) == 0x180U) ||
+            ((reg & 0xF00U) == 0x300U) ||
+            ((reg & 0xF00U) == 0xC00U) ||
+            ((reg & 0xF00U) == 0xD00U) ||
+            ((reg & 0xF00U) == 0xE00U) ||
+            ((reg & 0xF00U) == 0x600U) ||
+            ((reg & 0xF00U) == 0x700U) ||
+            (reg == 0x1A0)  ||
+            (reg == 0x0CE)  ||
+            (reg == 0x19C)  ||
+            (reg == 0x1A2)  ||
+            (reg == 0x1AD)  ||
+            (reg == 0x1A6))
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;
+    }
+}
 
 static int allowed_amd(uint32_t reg)
 {
@@ -190,6 +325,19 @@ static int allowed_amd15(uint32_t reg)
     }
 }
 
+static int allowed_amd16(uint32_t reg)
+{
+    if ( ((reg & 0xFFFFFFF0U) == 0xC0010000U) ||
+            ((reg & 0xFFFFFFF8U) == 0xC0010240U))
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;
+    }
+}
+
 static void msr_read(AccessDataRecord * dRecord)
 {
     uint64_t data;
@@ -199,14 +347,19 @@ static void msr_read(AccessDataRecord * dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
+    if (FD_MSR[cpu] == -2)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
     if (!allowed(reg))
     {
-        syslog(LOG_ERR, "attempt to read from restricted register %x", reg);
+        syslog(LOG_ERR, "attempt to read from restricted register 0x%x", reg);
         dRecord->errorcode = ERR_RESTREG;
         return;
     }
 
-    if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data)) 
+    if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
         syslog(LOG_ERR, "Failed to read data from msr device file on core %u", cpu);
         dRecord->errorcode = ERR_RWFAIL;
@@ -224,6 +377,12 @@ static void msr_write(AccessDataRecord * dRecord)
 
     dRecord->errorcode = ERR_NOERROR;
 
+    if (FD_MSR[cpu] == -2)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+
     if (!allowed(reg))
     {
         syslog(LOG_ERR, "attempt to write to restricted register %x", reg);
@@ -231,9 +390,9 @@ static void msr_write(AccessDataRecord * dRecord)
         return;
     }
 
-    if (pwrite(FD_MSR[cpu], &data, sizeof data, reg) != sizeof(data)) 
+    if (pwrite(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to write data from msr device file on core %u", cpu);
+        syslog(LOG_ERR, "Failed to write data to msr device file on core %u", cpu);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -249,7 +408,12 @@ static void pci_read(AccessDataRecord* dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
-    if ( !FD_PCI[socketId][device] )
+    if (FD_PCI[socketId][device] == -2)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+    else if ( !FD_PCI[socketId][device] )
     {
         strncpy(pci_filepath, PCI_ROOT_PATH, 30);
         strncat(pci_filepath, socket_bus[socketId], 10);
@@ -259,19 +423,19 @@ static void pci_read(AccessDataRecord* dRecord)
 
         if ( FD_PCI[socketId][device] < 0)
         {
-            syslog(LOG_ERR, "Failed to open device file on socket %u", socketId);
+            syslog(LOG_ERR, "Failed to open device file %s on socket %u", pci_filepath, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
     }
 
-    if ( pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data)) 
+    if ( pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to read data from pci device file on socket %u", socketId);
+        syslog(LOG_ERR, "Failed to read data from pci device file on socket %u device %u",
+                socketId, device);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
-    //    printf("READ Device %s cpu %d reg 0x%x data 0x%x \n",bdata(filepath), cpu, reg, data);
 
     dRecord->data = (uint64_t) data;
 }
@@ -286,8 +450,12 @@ static void pci_write(AccessDataRecord* dRecord)
     uint32_t data = (uint32_t) dRecord->data;
 
     dRecord->errorcode = ERR_NOERROR;
-
-    if ( !FD_PCI[socketId][device] )
+    if (FD_PCI[socketId][device] == -2)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+    else if ( !FD_PCI[socketId][device] )
     {
         strncpy(pci_filepath, PCI_ROOT_PATH, 30);
         strncat(pci_filepath, socket_bus[socketId], 10);
@@ -297,15 +465,15 @@ static void pci_write(AccessDataRecord* dRecord)
 
         if ( FD_PCI[socketId][device] < 0)
         {
-            syslog(LOG_ERR, "Failed to open device file on socket %u", socketId);
+            syslog(LOG_ERR, "Failed to open device file %s on socket %u", pci_filepath, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
     }
 
-    if (pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data) 
+    if (pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
     {
-        syslog(LOG_ERR, "Failed to write data from pci device file on socket %u", socketId);
+        syslog(LOG_ERR, "Failed to write data to pci device file on socket %u", socketId);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -316,7 +484,7 @@ static void kill_client(void)
 {
     syslog(LOG_NOTICE, "daemon dropped client");
 
-    if (connfd != -1) 
+    if (connfd != -1)
     {
         CHECK_ERROR(close(connfd), socket close failed);
     }
@@ -329,14 +497,9 @@ static void stop_daemon(void)
     kill_client();
     syslog(LOG_NOTICE, "daemon exiting");
 
-    if (sockfd != -1) 
+    if (sockfd != -1)
     {
         CHECK_ERROR(close(sockfd), socket close sockfd failed);
-
-        if (sysdaemonmode) 
-        {
-            CHECK_ERROR(unlink(filepath), unlink of socket failed);
-        }
     }
 
     free(filepath);
@@ -349,19 +512,11 @@ static void Signal_Handler(int sig)
     if (sig == SIGPIPE)
     {
         syslog(LOG_NOTICE, "SIGPIPE? client crashed?!");
-
-        if (sysdaemonmode) 
-        {
-            kill_client();
-        }
-        else
-        {
-            stop_daemon();
-        }
+        stop_daemon();
     }
 
     /* For SIGALRM we just return - we're just here to create a EINTR */
-    if ((sig == SIGTERM) && (sysdaemonmode))
+    if ((sig == SIGTERM))
     {
         stop_daemon();
     }
@@ -388,7 +543,7 @@ static void daemonize(int* parentPid)
     /* If we got a good PID, then we can exit the parent process. */
     if (pid > 0)
     {
-        exit(EXIT_SUCCESS);
+        exit(ERR_NOERROR);
     }
 
     /* At this point we are executing as the child process */
@@ -404,7 +559,7 @@ static void daemonize(int* parentPid)
 
     /* Change the current working directory.  This prevents the current
        directory from being locked; hence not being able to remove it. */
-    if ((chdir("/")) < 0) 
+    if ((chdir("/")) < 0)
     {
         syslog(LOG_ERR, "chdir failed:  %s", strerror(errno));
         exit(EXIT_FAILURE);
@@ -420,7 +575,7 @@ static void daemonize(int* parentPid)
 
 /* #####  MAIN FUNCTION DEFINITION   ################## */
 
-int main(int argc, char ** argv) 
+int main(void)
 {
     int ret;
     pid_t pid;
@@ -428,97 +583,95 @@ int main(int argc, char ** argv)
     socklen_t socklen;
     AccessDataRecord dRecord;
     mode_t oldumask;
-    static struct option long_options[] = {
-        { "help",      0, 0, '?' },
-        { "sysdaemon", 0, 0, 's' },
-        { 0, 0, 0, 0 }
-    };
-    int c;
     uint32_t numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
+    uint32_t model;
+    int isIntel = 1;
 
-    while ((c = getopt_long(argc, argv, "h?", long_options, NULL)) > 0) {
-        switch (c) {
-            case '?':
-            case 'h':
-                printf("Syntax: %s --sysdaemon\n", argv[0]);
-                printf("You should NOT manually call this program unless you");
-                printf(" want to run it as a system daemon.\n");
-                exit(EXIT_SUCCESS);
-                break;
-            case 's':
-#if (ALLOWSYSDAEMONMODE == 1)
-                if (getuid() != 0) {
-                    printf("Sorry, only root is allowed to call sysdaemonmode.\n");
-                    exit(EXIT_FAILURE);
-                }
-                sysdaemonmode = 1;
-#else
-                printf("sysdaemonmode has been disallowed at compiletime.\n");
-                exit(EXIT_FAILURE);
-#endif
-                break;
-            default:
-                fprintf(stderr, "Invalid option. try -h for help.\n");
-                exit(EXIT_FAILURE);
-        };
+    if (!lock_check())
+    {
+        fprintf(stderr,"Access to performance counters is locked. Exiting!\n");
+        exit(EXIT_FAILURE);
+    }
+
+    for ( uint32_t i=0; i < numHWThreads; i++ )
+    {
+        FD_MSR[i] = -1;
+    }
+
+    uint32_t  eax = 0x00;
+    uint32_t  ebx = 0x00;
+    
+    CPUID;
+    if (ebx == 0x68747541U)
+    {
+        isIntel = 0;
+    }
+
+    eax = 0x01;
+    CPUID;
+    uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
+    model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
+
+    switch (family)
+    {
+        case P6_FAMILY:
+            allowed = allowed_intel;
+
+            if (isIntel && ((model == SANDYBRIDGE)    ||
+                            (model == SANDYBRIDGE_EP) ||
+                            (model == IVYBRIDGE)      ||
+                            (model == IVYBRIDGE_EP) ))
+            {
+                allowed = allowed_sandybridge;
+                isPCIUncore = 1;
+            }
+            else if (isIntel && ((model == HASWELL)    ||
+                                 (model == HASWELL_M1) ||
+                                 (model == HASWELL_M2) ||
+                                 (model == HASWELL_EX)))
+            {
+                allowed = allowed_haswell;
+            }
+            else if (isIntel && (model == ATOM_SILVERMONT))
+            {
+                allowed = allowed_silvermont;
+            }
+            else if (isIntel && (model == WESTMERE_EX))
+            {
+                allowed = allowed_westmereEX;
+            }
+            break;
+        case K8_FAMILY:
+        case K10_FAMILY:
+            if (!isIntel) 
+            {
+                allowed = allowed_amd;
+            }
+            break;
+        case K15_FAMILY:
+            if (!isIntel) 
+            {
+                allowed = allowed_amd15;
+            }
+            break;
+        case K16_FAMILY:
+            if (!isIntel) 
+            {
+                allowed = allowed_amd16;
+            }
+            break;
+        default:
+            fprintf(stderr, "ERROR - [%s:%d] - Unsupported processor. Exiting!\n",
+                    __FILE__, __LINE__);
+            exit(EXIT_FAILURE);
     }
 
     openlog(ident, 0, LOG_USER);
     daemonize(&pid);
 
-    {
-        uint32_t  eax = 0x00;
-        uint32_t  ebx = 0x00;
-        int isIntel = 1;
-        CPUID;
-        if (ebx == 0x68747541U)
-        {
-            isIntel = 0;
-        }
-
-        eax = 0x01;
-        CPUID;
-        uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
-        uint32_t model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
-
-        switch (family)
-        {
-            case P6_FAMILY:
-                allowed = allowed_intel;
-
-                if ((model == SANDYBRIDGE) || (model == SANDYBRIDGE_EP))
-                {
-                    allowed = allowed_sandybridge;
-                    isSandyBridge = 1;
-                }
-                break;
-            case K8_FAMILY:
-                if (isIntel)
-                {
-                    syslog(LOG_ERR,
-                            "ERROR - [%s:%d] - Netburst architecture is not supported! Exiting! \n",
-                            __FILE__,__LINE__);
-                    exit(EXIT_FAILURE);
-                }
-            case K10_FAMILY:
-                allowed = allowed_amd;
-                break;
-            case K15_FAMILY:
-                allowed = allowed_amd15;
-                break;
-            default:
-                syslog(LOG_ERR, "ERROR - [%s:%d] - Unsupported processor. Exiting!  \n", __FILE__, __LINE__);
-                exit(EXIT_FAILURE);
-        }
-    }
-
     /* setup filename for socket */
     filepath = (char*) calloc(sizeof(addr1.sun_path), 1);
-    if (sysdaemonmode == 1) {
-        snprintf(filepath, sizeof(addr1.sun_path), "%s", SYSDAEMONSOCKETPATH);
-    } else {
-        snprintf(filepath, sizeof(addr1.sun_path), "/tmp/likwid-%d", pid);
-    }
+    snprintf(filepath, sizeof(addr1.sun_path), "/tmp/likwid-%d", pid);
 
     /* get a socket */
     EXIT_IF_ERROR(sockfd = socket(AF_LOCAL, SOCK_STREAM, 0), socket failed);
@@ -556,271 +709,192 @@ int main(int argc, char ** argv)
 
     syslog(LOG_NOTICE, "daemon started");
 
-    if (sysdaemonmode == 0) 
+    /* setup an alarm to stop the daemon if there is no connect.*/
+    alarm(15U);
+
+    if ((connfd = accept(sockfd, (SA*) &addr1, &socklen)) < 0)
     {
-        /* The normal case: without sysdaemon mode. We only ever accept one client. */
-        /* accept one connect from one client */
-
-        /* setup an alarm to stop the daemon if there is no connect.
-         * or to change to different metrics in sysdaemonmode. */
-        alarm(15U);
-
-        if ((connfd = accept(sockfd, (SA*) &addr1, &socklen)) < 0)
+        if (errno == EINTR)
         {
-            if (errno == EINTR)
-            {
-                syslog(LOG_ERR, "exiting due to timeout - no client connected after 15 seconds.");
-            }
-            else
-            {
-                syslog(LOG_ERR, "accept() failed:  %s", strerror(errno));
-            }
-            CHECK_ERROR(unlink(filepath), unlink of socket failed);
-            exit(EXIT_FAILURE);
+            syslog(LOG_ERR, "exiting due to timeout - no client connected after 15 seconds.");
         }
-
-        alarm(0);
-        CHECK_ERROR(unlink(filepath), unlink of socket failed);
-        syslog(LOG_NOTICE, "daemon accepted client");
-
+        else
         {
-            char* msr_file_name = (char*) malloc(MAX_PATH_LENGTH * sizeof(char));
+            syslog(LOG_ERR, "accept() failed:  %s", strerror(errno));
+        }
+        CHECK_ERROR(unlink(filepath), unlink of socket failed);
+        exit(EXIT_FAILURE);
+    }
 
-            /* Open MSR device files for less overhead.
-             * NOTICE: This assumes consecutive processor Ids! */
-            for ( uint32_t i=0; i < numHWThreads; i++ )
+    alarm(0);
+    CHECK_ERROR(unlink(filepath), unlink of socket failed);
+    syslog(LOG_NOTICE, "daemon accepted client");
+
+    {
+        char* msr_file_name = (char*) malloc(MAX_PATH_LENGTH * sizeof(char));
+
+        /* Open MSR device files for less overhead.
+         * NOTICE: This assumes consecutive processor Ids! */
+        for ( uint32_t i=0; i < numHWThreads; i++ )
+        {
+#ifdef __MIC
+            sprintf(msr_file_name,"/dev/msr%d",i);
+            if (access(msr_file_name, F_OK))
             {
                 sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
-                FD_MSR[i] = open(msr_file_name, O_RDWR);
+            }
+#else
+            sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
+#endif
+            FD_MSR[i] = open(msr_file_name, O_RDWR);
 
-                if ( FD_MSR[i] < 0 )
+            if ( FD_MSR[i] < 0 )
+            {
+                syslog(LOG_ERR, "Failed to open device file %s.",msr_file_name);
+                FD_MSR[i] = -2;
+            }
+        }
+
+        free(msr_file_name);
+
+        if (isPCIUncore)
+        {
+            for (int j=0; j<MAX_NUM_NODES; j++)
+            {
+                socket_bus[j] = "N-A";
+                for (int i=0; i<MAX_NUM_DEVICES; i++)
                 {
-                    syslog(LOG_ERR, "Failed to open device files.");
+                    FD_PCI[j][i] = -2;
                 }
             }
 
-            free(msr_file_name);
+            /* determine PCI-BUSID mapping ... */
+            FILE *fptr;
+            char buf[1024];
+            uint32_t testDevice;
+            uint32_t sbus, sdevfn, svend;
+            int cntr = 0;
+            int socket_count = 0;
 
-            if (isSandyBridge)
+            if (model == SANDYBRIDGE_EP)
             {
-                for (int j=0; j<MAX_NUM_NODES; j++)
+                testDevice = 0x80863c44;
+            }
+            else if (model == IVYBRIDGE_EP)
+            {
+                testDevice = 0x80860e36;
+            }
+            else
+            {
+                testDevice = 0;
+                syslog(LOG_NOTICE, "PCI Uncore not supported on this system");
+            }
+
+            if ( ((fptr = fopen("/proc/bus/pci/devices", "r")) == NULL) || !testDevice)
+            {
+                syslog(LOG_NOTICE, "Unable to open /proc/bus/pci/devices");
+            }
+            else
+            {
+                while( fgets(buf, sizeof(buf)-1, fptr) )
+                {
+                    if ( sscanf(buf, "%2x%2x %8x", &sbus, &sdevfn, &svend) == 3 &&
+                            svend == testDevice )
+                    {
+                        socket_bus[cntr] = (char*)malloc(4);
+                        sprintf(socket_bus[cntr++], "%02x/", sbus);
+                    }
+                }
+                fclose(fptr);
+            }
+
+            if ( cntr == 0 )
+            {
+                syslog(LOG_NOTICE, "Uncore not supported on this system");
+            }
+            else
+            {
+                socket_count = cntr;
+
+                for (int j=0; j<socket_count; j++)
                 {
                     for (int i=0; i<MAX_NUM_DEVICES; i++)
                     {
-                        FD_PCI[j][i] = 0;
-                    }
-                }
+                        sprintf(pci_filepath, "%s%s%s",PCI_ROOT_PATH,socket_bus[j],pci_DevicePath[i]);
 
-                /* TODO Exten to 4 socket systems */
-#if 0
-                if ( cpuid_topology.numSockets == 2 )
-                {
-                    /* Already correctly initialized */
-                }
-                else if ( cpuid_topology.numSockets == 4 )
-                {
-                    strcpy(socket_bus[1],"3f/");
-                }
-                else 
-                {
-                    /*TODO Check devices on single socket variants!! */
-                    syslog(LOG_NOTICE, "Uncore currently not supported for single socket systems");
-                }
-#endif
-            }
-        }
-
-        while (1)
-        {
-            ret = read(connfd, (void*) &dRecord, sizeof(AccessDataRecord));
-
-            if (ret < 0)
-            { 
-                syslog(LOG_ERR, "ERROR - [%s:%d] read from client failed  - %s \n", __FILE__, __LINE__, strerror(errno));
-                stop_daemon();
-            }
-            else if (ret == 0)
-            {
-                syslog(LOG_ERR, "ERROR - [%s:%d] zero read", __FILE__, __LINE__);
-                stop_daemon();
-            }
-            else if (ret != sizeof(AccessDataRecord))
-            {
-                syslog(LOG_ERR, "ERROR - [%s:%d] unaligned read", __FILE__, __LINE__);
-                stop_daemon();
-            }
-
-
-            if (dRecord.type == DAEMON_READ)
-            {
-                if (dRecord.device == DAEMON_AD_MSR)
-                {
-                    msr_read(&dRecord);
-                }
-                else
-                {
-                    pci_read(&dRecord);
-                }
-            }
-            else if (dRecord.type == DAEMON_WRITE)
-            {
-                if (dRecord.device == DAEMON_AD_MSR)
-                {
-                    msr_write(&dRecord);
-                    dRecord.data = 0x0ULL;
-                }
-                else
-                {
-                    pci_write(&dRecord);
-                    dRecord.data = 0x0ULL;
-                }
-            }
-            else if (dRecord.type == DAEMON_EXIT)
-            {
-                stop_daemon();
-            }
-            else
-            {
-                syslog(LOG_ERR, "unknown daemon access type  %d", dRecord.type);
-                dRecord.errorcode = ERR_UNKNOWN;
-            }
-
-            //    syslog(LOG_NOTICE, "write: cpu %d reg 0x%x data 0x%x type %d device %d error %d \n",
-            //	    dRecord.cpu, dRecord.reg, dRecord.data, dRecord.type, dRecord.device, dRecord.errorcode);
-
-            EXIT_IF_ERROR(write(connfd, (void*) &dRecord, sizeof(AccessDataRecord)), write failed);
-        }
-    } else {  /* Sysdaemonmode. */
-        /* Things are slightly more complicated in sysdaemonmode: We can accept
-         * multiple clients, and clients can preemt lower priority clients.
-         * While this does share quite a bit of copy&paste with above, it is
-         * seperated intentionally, to keep the code for the setuid use case
-         * above as simple as possible. */
-        int haveclient = 0;
-        int clientprio = 0;
-        int numberOfConnects = 0;
-        fd_set fds;
-
-        while (1)
-        {
-            FD_ZERO(&fds);
-            FD_SET(sockfd, &fds);
-
-            if (haveclient) 
-            {
-                for (int i=0; i<(numberOfConnects); i++)
-                {
-                    if (CONNFD[i]) 
-                    {
-                        FD_SET(CONNFD[i], &fds);
-                    }
-                }
-            }
-
-            if (select(FD_SETSIZE, &fds, NULL, NULL, NULL) < 0) 
-            {
-                syslog(LOG_ERR, "exiting due to error on select().");
-                exit(EXIT_FAILURE);
-            }
-
-            if (haveclient) 
-            {
-                if (FD_ISSET(connfd, &fds)) /* data (or an error) from our client */
-                {
-                    ret = read(connfd, (void*) &dRecord, sizeof(AccessDataRecord));
-
-                    if (ret < 0)
-                    {
-                        syslog(LOG_ERR, "ERROR - [%s:%d] read from client failed  - %s \n", __FILE__, __LINE__, strerror(errno));
-                        kill_client();
-                        haveclient = 0;
-                    }
-                    else if (ret == 0)
-                    {
-                        syslog(LOG_ERR, "ERROR - [%s:%d] zero read", __FILE__, __LINE__);
-                        kill_client();
-                        haveclient = 0;
-                    }
-                    else if (ret != sizeof(AccessDataRecord))
-                    {
-                        syslog(LOG_ERR, "ERROR - [%s:%d] unaligned read", __FILE__, __LINE__);
-                        kill_client();
-                        haveclient = 0;
-                    }
-
-                    if (dRecord.type == DAEMON_READ)
-                    {
-                        msr_read(&dRecord);
-                    }
-                    else if (dRecord.type == DAEMON_WRITE)
-                    {
-                        msr_write(&dRecord);
-                        dRecord.data = 0x0ULL;
-                    }
-                    else if (dRecord.type == DAEMON_MARK_CLIENT_LOWPRIO)
-                    {
-                        clientprio = 20;
-                    }
-                    else if (dRecord.type == DAEMON_EXIT)
-                    {
-                        kill_client();
-                        haveclient = 0;
-                        clientprio = 0;
-                    }
-                    else
-                    {
-                        syslog(LOG_ERR, "unknown msr access type  %d", dRecord.type);
-                        dRecord.errorcode = ERR_UNKNOWN;
-                    }
-                    if (connfd > 0) /* Still >0? we might have lost the client above. */
-                    { 
-                        if (write(connfd, (void*) &dRecord, sizeof(AccessDataRecord)) < 0)
+                        if (!access(pci_filepath,F_OK))
                         {
-                            syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno));
-                            kill_client();
-                        }
-                    }
-                }
-            }
-
-            if (FD_ISSET(sockfd, &fds)) /* new connection */
-            { 
-                int newconnfd = accept(sockfd, (SA*) &addr1, &socklen);
-
-                if (newconnfd < 0) 
-                {
-                    syslog(LOG_ERR, "error on accept() - ignoring");
-                }
-                else 
-                {
-                    if (haveclient) /* Check if the old client can be dropped */
-                    { 
-                        if (clientprio > 0)
-                        {
-                            kill_client();
-                            /* FIXME reset MSR state? */
-                            connfd = newconnfd;
-                            clientprio = 0;
+                            FD_PCI[j][i] = 0;
                         }
                         else
-                        { /* it cannot, so drop the new client. */
-                            memset(&dRecord, 0, sizeof(dRecord));
-                            dRecord.errorcode = ERR_DAEMONBUSY;
-                            EXIT_IF_ERROR(write(newconnfd, (void*) &dRecord, sizeof(AccessDataRecord)), write failed);
-                            EXIT_IF_ERROR(close(newconnfd), close failed);
+                        {
+                            syslog(LOG_NOTICE, "Device %s not found, excluded it from device list\n",pci_filepath);
                         }
-                    } 
-                    else
-                    {
-                        connfd = newconnfd;
-                        haveclient = 1;
-                        clientprio = 0;
                     }
                 }
             }
         }
     }
 
+    while (1)
+    {
+        ret = read(connfd, (void*) &dRecord, sizeof(AccessDataRecord));
+
+        if (ret < 0)
+        {
+            syslog(LOG_ERR, "ERROR - [%s:%d] read from client failed  - %s \n",
+                    __FILE__, __LINE__, strerror(errno));
+            stop_daemon();
+        }
+        else if (ret == 0)
+        {
+            syslog(LOG_ERR, "ERROR - [%s:%d] zero read", __FILE__, __LINE__);
+            stop_daemon();
+        }
+        else if (ret != sizeof(AccessDataRecord))
+        {
+            syslog(LOG_ERR, "ERROR - [%s:%d] unaligned read", __FILE__, __LINE__);
+            stop_daemon();
+        }
+
+
+        if (dRecord.type == DAEMON_READ)
+        {
+            if (dRecord.device == DAEMON_AD_MSR)
+            {
+                msr_read(&dRecord);
+            }
+            else
+            {
+                pci_read(&dRecord);
+            }
+        }
+        else if (dRecord.type == DAEMON_WRITE)
+        {
+            if (dRecord.device == DAEMON_AD_MSR)
+            {
+                msr_write(&dRecord);
+                dRecord.data = 0x0ULL;
+            }
+            else
+            {
+                pci_write(&dRecord);
+                dRecord.data = 0x0ULL;
+            }
+        }
+        else if (dRecord.type == DAEMON_EXIT)
+        {
+            stop_daemon();
+        }
+        else
+        {
+            syslog(LOG_ERR, "unknown daemon access type  %d", dRecord.type);
+            dRecord.errorcode = ERR_UNKNOWN;
+        }
+
+        EXIT_IF_ERROR(write(connfd, (void*) &dRecord, sizeof(AccessDataRecord)), write failed);
+    }
+
     /* never reached */
     return EXIT_SUCCESS;
 }
