@@ -6,13 +6,13 @@
  *      Description:  A application to determine the thread and cache topology
  *                    on x86 processors.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
 #include <ctype.h>
 
 #include <types.h>
+#include <error.h>
 #include <cpuid.h>
 #include <timer.h>
 #include <affinity.h>
@@ -50,23 +51,26 @@
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define HELP_MSG \
-printf("\nlikwid-topology --  Version %d.%d \n\n",VERSION,RELEASE); \
-printf("A tool to print the thread and cache topology on x86 CPUs.\n"); \
-printf("Options:\n"); \
-printf("-h\t Help message\n"); \
-printf("-v\t Version information\n"); \
-printf("-c\t list cache information\n"); \
-printf("-C\t measure processor clock\n"); \
-printf("-o\t Store output to file. (Optional: Apply text filter)\n"); \
-printf("-g\t graphical output\n\n")
+    fprintf(OUTSTREAM, "\nlikwid-topology --  Version %d.%d \n\n",VERSION,RELEASE); \
+    fprintf(OUTSTREAM, "A tool to print the thread and cache topology on x86 CPUs.\n"); \
+    fprintf(OUTSTREAM, "Options:\n"); \
+    fprintf(OUTSTREAM, "-h\t Help message\n"); \
+    fprintf(OUTSTREAM, "-v\t Version information\n"); \
+    fprintf(OUTSTREAM, "-c\t list cache information\n"); \
+    fprintf(OUTSTREAM, "-C\t measure processor clock\n"); \
+    fprintf(OUTSTREAM, "-o\t Store output to file, with output conversion according to file suffix\n"); \
+    fprintf(OUTSTREAM, "\t Conversion scripts can be supplied in %s\n",TOSTRING(LIKWIDFILTERPATH)); \
+    fprintf(OUTSTREAM, "-g\t graphical output\n\n"); \
+    fflush(OUTSTREAM);
 
 #define VERSION_MSG \
-printf("likwid-topology  %d.%d \n\n",VERSION,RELEASE)
+    fprintf(OUTSTREAM, "likwid-topology  %d.%d \n\n",VERSION,RELEASE); \
+    fflush(OUTSTREAM);
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 int main (int argc, char** argv)
-{ 
+{
     int optGraphical = 0;
     int optCaches = 0;
     int optClock = 0;
@@ -86,10 +90,10 @@ int main (int argc, char** argv)
         {
             case 'h':
                 HELP_MSG;
-                exit (EXIT_SUCCESS);    
+                exit (EXIT_SUCCESS);
             case 'v':
                 VERSION_MSG;
-                exit (EXIT_SUCCESS);    
+                exit (EXIT_SUCCESS);
             case 'g':
                 optGraphical = 1;
                 break;
@@ -126,21 +130,24 @@ int main (int argc, char** argv)
                 return EXIT_FAILURE;
             default:
                 HELP_MSG;
-                exit (EXIT_SUCCESS);    
+                exit (EXIT_SUCCESS);
         }
     }
 
-    cpuid_init();
+    if (cpuid_init() == EXIT_FAILURE)
+    {
+        ERROR_PLAIN_PRINT(Unsupported processor!);
+    }
     affinity_init();
     numa_init();
 
     fprintf(OUTSTREAM, HLINE);
-    fprintf(OUTSTREAM, "CPU type:\t%s \n",cpuid_info.name);
+    fprintf(OUTSTREAM, "CPU type:\t%s\n",cpuid_info.name);
 
     if (optClock)
     {
         timer_init();
-        fprintf(OUTSTREAM, "CPU clock:\t%3.2f GHz \n",  (float) timer_getCpuClock() * 1.E-09);
+        fprintf(OUTSTREAM, "CPU clock:\t%3.2f GHz\n",  (float) timer_getCpuClock() * 1.E-09);
     }
 
     /*----------------------------------------------------------------------
@@ -185,6 +192,7 @@ int main (int argc, char** argv)
         fprintf(OUTSTREAM, ")\n");
     }
     fprintf(OUTSTREAM, HLINE"\n");
+    fflush(OUTSTREAM);
 
     /*----------------------------------------------------------------------
      *  Cache Topology
@@ -203,7 +211,7 @@ int main (int argc, char** argv)
                 fprintf(OUTSTREAM, "Size:\t%d kB\n",
                         cpuid_topology.cacheLevels[i].size/1024);
             }
-            else 
+            else
             {
                 fprintf(OUTSTREAM, "Size:\t%d MB\n",
                         cpuid_topology.cacheLevels[i].size/1048576);
@@ -231,7 +239,7 @@ int main (int argc, char** argv)
                         cpuid_topology.cacheLevels[i].associativity);
                 fprintf(OUTSTREAM, "Number of sets:\t%d\n",
                         cpuid_topology.cacheLevels[i].sets);
-                fprintf(OUTSTREAM, "Cache line size:%d\n",
+                fprintf(OUTSTREAM, "Cache line size:\t%d\n",
                         cpuid_topology.cacheLevels[i].lineSize);
                 if(cpuid_topology.cacheLevels[i].inclusive)
                 {
@@ -284,6 +292,7 @@ int main (int argc, char** argv)
     }
 
     fprintf(OUTSTREAM, "\n");
+    fflush(OUTSTREAM);
 
     /*----------------------------------------------------------------------
      *  NUMA Topology
@@ -326,6 +335,7 @@ int main (int argc, char** argv)
         }
     }
     fprintf(OUTSTREAM, "\n");
+    fflush(OUTSTREAM);
 
     /*----------------------------------------------------------------------
      *  Graphical topology
@@ -340,7 +350,7 @@ int main (int argc, char** argv)
         fprintf(OUTSTREAM, SLINE);
 
         /* Allocate without instruction cache */
-        if ( cpuid_info.family == P6_FAMILY || cpuid_info.family == MIC_FAMILY ) 
+        if ( cpuid_info.family == P6_FAMILY || cpuid_info.family == MIC_FAMILY )
         {
             container = asciiBoxes_allocateContainer(
                     cpuid_topology.numCacheLevels,
@@ -379,7 +389,7 @@ int main (int argc, char** argv)
                     tmp++;
                     threadNode = tree_getNextNode(threadNode);
                 }
-                asciiBoxes_addBox(container, 0, j, boxLabel); 
+                asciiBoxes_addBox(container, 0, j, boxLabel);
                 j++;
                 coreNode = tree_getNextNode(coreNode);
             }
@@ -417,7 +427,7 @@ int main (int argc, char** argv)
                                 boxLabel = bformat("%dkB",
                                         cpuid_topology.cacheLevels[i].size/1024);
                             }
-                            else 
+                            else
                             {
                                 boxLabel = bformat("%dMB",
                                         cpuid_topology.cacheLevels[i].size/1048576);
@@ -438,17 +448,17 @@ int main (int argc, char** argv)
                                         lineCursor,
                                         columnCursor,
                                         columnCursor+cacheWidth,
-                                        boxLabel); 
+                                        boxLabel);
 
                                 columnCursor += sharedCores;
                             }
-                            else 
+                            else
                             {
                                 asciiBoxes_addBox(
                                         container,
                                         lineCursor,
                                         columnCursor,
-                                        boxLabel); 
+                                        boxLabel);
 
                                 columnCursor++;
                             }
@@ -459,7 +469,7 @@ int main (int argc, char** argv)
                 }
             }
 
-            asciiBoxes_print(container);
+            asciiBoxes_print(OUTSTREAM, container);
             socketNode = tree_getNextNode(socketNode);
         }
         bdestroy(boxLabel);
@@ -470,7 +480,23 @@ int main (int argc, char** argv)
     /* call filterscript if specified */
     if (!biseqcstr(filterScript,"NO"))
     {
+        struct bstrList* tokens;
+        tokens = bsplit(filterScript,' ');
+        if (access(bdata(tokens->entry[0]), F_OK))
+        {
+            fprintf(stderr, "Cannot find filter %s!\n", bdata(tokens->entry[0]));
+            bstrListDestroy(tokens);
+            exit(EXIT_FAILURE);
+        }
+        if (access(bdata(tokens->entry[0]), X_OK))
+        {
+            fprintf(stderr, "Cannot execute filter %s!\n", bdata(tokens->entry[0]));
+            bstrListDestroy(tokens);
+            exit(EXIT_FAILURE);
+        }
+        bstrListDestroy(tokens);
         bcatcstr(filterScript, " topology");
+
         if (system(bdata(filterScript)) == EOF)
         {
             fprintf(stderr, "Failed to execute filter %s!\n", bdata(filterScript));
