@@ -6,13 +6,13 @@
  *      Description:  Implementation of cpuid module.
  *                  Provides API to extract cpuid info on x86 processors.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,7 @@
 #include <unistd.h>
 #include <sched.h>
 #include <time.h>
+#include <math.h>
 
 #include <error.h>
 #include <cpuid.h>
@@ -55,7 +56,6 @@ CpuTopology cpuid_topology;
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int largest_function = 0;
-static int likwid_inCPUSet  = 0;
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
@@ -76,12 +76,20 @@ static char* pentium_m_d_str = "Intel Pentium M Dothan processor";
 static char* core_duo_str = "Intel Core Duo processor";
 static char* core_2a_str = "Intel Core 2 65nm processor";
 static char* core_2b_str = "Intel Core 2 45nm processor";
+static char* atom_45_str = "Intel Atom 45nm processor";
+static char* atom_32_str = "Intel Atom 32nm processor";
+static char* atom_22_str = "Intel Atom 22nm processor";
+static char* atom_silvermont_str = "Intel Atom (Silvermont) 22nm processor";
+static char* atom_saltwell_str = "Intel Atom (Saltwell) 32nm processor";
 static char* nehalem_bloom_str = "Intel Core Bloomfield processor";
 static char* nehalem_lynn_str = "Intel Core Lynnfield processor";
 static char* nehalem_west_str = "Intel Core Westmere processor";
 static char* sandybridge_str = "Intel Core SandyBridge processor";
 static char* ivybridge_str = "Intel Core IvyBridge processor";
+static char* ivybridge_ep_str = "Intel Core IvyBridge EP processor";
 static char* sandybridge_ep_str = "Intel Core SandyBridge EP processor";
+static char* haswell_str = "Intel Core Haswell processor";
+static char* haswell_ex_str = "Intel Core Haswell EX processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
@@ -91,6 +99,7 @@ static char* shanghai_str = "AMD Shanghai processor";
 static char* istanbul_str = "AMD Istanbul processor";
 static char* magnycours_str = "AMD Magny Cours processor";
 static char* interlagos_str = "AMD Interlagos processor";
+static char* kabini_str = "AMD Family 16 model - Kabini processor";
 static char* opteron_sc_str = "AMD Opteron single core 130nm processor";
 static char* opteron_dc_e_str = "AMD Opteron Dual Core Rev E 90nm processor";
 static char* opteron_dc_f_str = "AMD Opteron Dual Core Rev F 90nm processor";
@@ -107,11 +116,55 @@ static uint32_t eax, ebx, ecx, edx;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
+static void initTopology(FILE* file)
+{
+    size_t items;
+    HWThread* hwThreadPool;
+    CacheLevel* cacheLevels;
+    TreeNode* currentNode;
+
+    items = fread((void*) &cpuid_topology, sizeof(CpuTopology), 1, file);
+
+    hwThreadPool = (HWThread*) malloc(cpuid_topology.numHWThreads * sizeof(HWThread));
+    items = fread((void*) hwThreadPool, sizeof(HWThread), cpuid_topology.numHWThreads, file);
+    cpuid_topology.threadPool = hwThreadPool;
+
+    cacheLevels = (CacheLevel*) malloc(cpuid_topology.numCacheLevels * sizeof(CacheLevel));
+    items = fread((void*) cacheLevels, sizeof(CacheLevel), cpuid_topology.numCacheLevels, file);
+    cpuid_topology.cacheLevels = cacheLevels;
+    cpuid_topology.topologyTree = NULL;
+
+    tree_init(&cpuid_topology.topologyTree, 0);
+
+    for (uint32_t i=0; i<  cpuid_topology.numHWThreads; i++)
+    {
+        if (!tree_nodeExists(cpuid_topology.topologyTree,
+                    hwThreadPool[i].packageId))
+        {
+            tree_insertNode(cpuid_topology.topologyTree,
+                    hwThreadPool[i].packageId);
+        }
+        currentNode = tree_getNode(cpuid_topology.topologyTree,
+                hwThreadPool[i].packageId);
+
+        if (!tree_nodeExists(currentNode, hwThreadPool[i].coreId))
+        {
+            tree_insertNode(currentNode, hwThreadPool[i].coreId);
+        }
+        currentNode = tree_getNode(currentNode, hwThreadPool[i].coreId);
+
+        if (!tree_nodeExists(currentNode, i))
+        {
+            tree_insertNode(currentNode, i);
+        }
+    }
+}
+
 static uint32_t amdGetAssociativity(uint32_t flag)
 {
     uint32_t asso= 0;
 
-    switch ( flag ) 
+    switch ( flag )
     {
         case 0x0:
             asso = 0;
@@ -175,6 +228,9 @@ static int intelCpuidFunc_4(CacheLevel** cachePool)
     int maxNumLevels=0;
     uint32_t valid=1;
     CacheLevel* pool;
+    int threadsPerCpu = 0;
+    int numThreadsPerSocket = cpuid_topology.numCoresPerSocket *
+                              cpuid_topology.numThreadsPerCore;
 
     while (valid)
     {
@@ -208,6 +264,7 @@ static int intelCpuidFunc_4(CacheLevel** cachePool)
             pool[i].associativity *
             pool[i].lineSize;
         pool[i].threads = extractBitField(eax,10,14)+1;
+        pool[i].inclusive = edx&0x2;
 
         /* WORKAROUND cpuid reports wrong number of threads on SMT processor with SMT
          * turned off */
@@ -220,6 +277,11 @@ static int intelCpuidFunc_4(CacheLevel** cachePool)
                     (cpuid_info.model == SANDYBRIDGE) ||
                     (cpuid_info.model == SANDYBRIDGE_EP) ||
                     (cpuid_info.model == IVYBRIDGE) ||
+                    (cpuid_info.model == IVYBRIDGE_EP) ||
+                    (cpuid_info.model == HASWELL) ||
+                    (cpuid_info.model == HASWELL_EX) ||
+                    (cpuid_info.model == HASWELL_M1) ||
+                    (cpuid_info.model == HASWELL_M2) ||
                     (cpuid_info.model == WESTMERE_EX) ||
                     (cpuid_info.model == NEHALEM_EX))
             {
@@ -231,31 +293,69 @@ static int intelCpuidFunc_4(CacheLevel** cachePool)
         }
 
         /* :WORKAROUND:08/13/2009 08:34:15 AM:jt: For L3 caches the value is sometimes 
-         * too large in here. Ask Intel what is wrong here!
-         * Limit threads per Socket then to the maximum possible value.*/
-        if(pool[i].threads > (int)
-                (cpuid_topology.numCoresPerSocket*
-                 cpuid_topology.numThreadsPerCore))
+         * too large in here. 
+         * See Documentation: Threads contains maximum number of threads supported
+         * by the cache.
+         * Limit threads per Socket then to the maximum possible value. If the number
+         * of threads supported by the cache does not divide the threads on the socket
+         * without remainder, the threads are adjusted to fit the multiple caches.
+         */
+        if(pool[i].threads > numThreadsPerSocket)
         {
-            pool[i].threads = cpuid_topology.numCoresPerSocket*
-                cpuid_topology.numThreadsPerCore;
+            pool[i].threads = numThreadsPerSocket;
+        }
+        else if (((double)numThreadsPerSocket)/((double)pool[i].threads) != 
+                  (double)(numThreadsPerSocket/pool[i].threads))
+        {
+            pool[i].threads = numThreadsPerSocket/
+                (int)ceil(((double)numThreadsPerSocket)/((double)pool[i].threads));
+        }
+        /* For Intel Silvermont this is not enough. It returns 4 threads and 8 cores
+         * for the L2 cache. But according to the data sheet, each 1MB L2 cache slice 
+         * is shared by 2 threads/cores.
+         */
+        else if (pool[i].level == 2 && 
+                ((cpuid_info.model == ATOM_SILVERMONT_C) ||
+                 (cpuid_info.model == ATOM_SILVERMONT_E) ||
+                 (cpuid_info.model == ATOM_SILVERMONT_F1) ||
+                 (cpuid_info.model == ATOM_SILVERMONT_F2) ||
+                 (cpuid_info.model == ATOM_SILVERMONT_F3)))
+        {
+            pool[i].threads = 2;
         }
-        pool[i].inclusive = edx&0x2;
     }
 
+    
+
     return maxNumLevels;
 }
 
+static int recheck_numHWThreads()
+{
+    int cpucount = 0;
+    char line[1024];
+    FILE* fp = fopen("/proc/cpuinfo","r");
+    if (fp != NULL)
+    {
+        while( fgets(line,1024,fp) )
+        {
+            if (strncmp(line, "processor", 9) == 0)
+            {
+                cpucount++;
+            }
+        }
+    }
+    return cpucount;
+}
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void
-cpuid_init (void)
+int cpuid_init (void)
 {
     int isIntel = 1;
 
     /* FIXME: Race condition??? */
-    if (init) return;
+    if (init) return EXIT_SUCCESS;
     init =1;
 
     eax = 0x00;
@@ -273,10 +373,10 @@ cpuid_init (void)
     cpuid_info.model = (((eax>>16)&0xFU)<<4) + ((eax>>4)&0xFU);
     cpuid_info.stepping =  (eax&0xFU);
 
-    switch ( cpuid_info.family ) 
+    switch ( cpuid_info.family )
     {
         case P6_FAMILY:
-            switch ( cpuid_info.model ) 
+            switch ( cpuid_info.model )
             {
                 case PENTIUM_M_BANIAS:
                     cpuid_info.name = pentium_m_b_str;
@@ -324,6 +424,22 @@ cpuid_init (void)
                     cpuid_info.name = ivybridge_str;
                     break;
 
+                case IVYBRIDGE_EP:
+                    cpuid_info.name = ivybridge_ep_str;
+                    break;
+
+                case HASWELL:
+
+                case HASWELL_M1:
+
+                case HASWELL_M2:
+                    cpuid_info.name = haswell_str;
+                    break;
+
+                case HASWELL_EX:
+                    cpuid_info.name = haswell_ex_str;
+                    break;
+
                 case NEHALEM_EX:
                     cpuid_info.name = nehalem_ex_str;
                     break;
@@ -336,6 +452,28 @@ cpuid_init (void)
                     cpuid_info.name = xeon_mp_string;
                     break;
 
+                case ATOM_45:
+
+                case ATOM:
+                    cpuid_info.name = atom_45_str;
+                    break;
+
+                case ATOM_32:
+                    cpuid_info.name = atom_32_str;
+                    break;
+
+                case ATOM_22:
+                    cpuid_info.name = atom_22_str;
+                    break;
+
+                case ATOM_SILVERMONT_C:
+                case ATOM_SILVERMONT_E:
+                case ATOM_SILVERMONT_F1:
+                case ATOM_SILVERMONT_F2:
+                case ATOM_SILVERMONT_F3:
+                    cpuid_info.name = atom_silvermont_str;
+                    break;
+
                 default:
                     cpuid_info.name = unknown_intel_str;
                     break;
@@ -359,7 +497,7 @@ cpuid_init (void)
                 ERROR_PLAIN_PRINT(Netburst architecture is not supported);
             }
 
-            switch ( cpuid_info.model ) 
+            switch ( cpuid_info.model )
             {
                 case OPTERON_DC_E:
                     cpuid_info.name = opteron_dc_e_str;
@@ -403,7 +541,7 @@ cpuid_init (void)
             break;
 
         case K10_FAMILY:
-            switch ( cpuid_info.model ) 
+            switch ( cpuid_info.model )
             {
                 case BARCELONA:
                     cpuid_info.name = barcelona_str;
@@ -431,23 +569,114 @@ cpuid_init (void)
             cpuid_info.name = interlagos_str;
             break;
 
+        case K16_FAMILY:
+            cpuid_info.name = kabini_str;
+            break;
+            
         default:
-            ERROR_PLAIN_PRINT(Processor is not supported);
+            return EXIT_FAILURE;
             break;
     }
 
-    cpuid_info.features = (char*) malloc(100*sizeof(char));
-    if (edx & (1<<25)) strcpy(cpuid_info.features, "SSE ");
-    if (edx & (1<<26)) strcat(cpuid_info.features, "SSE2 ");
-    if (ecx & (1<<0))  strcat(cpuid_info.features, "SSE3 ");
-    if (ecx & (1<<19)) strcat(cpuid_info.features, "SSE4.1 ");
-    if (ecx & (1<<20)) strcat(cpuid_info.features, "SSE4.2 ");
-    if (ecx & (1<<12)) strcat(cpuid_info.features, "FMA ");
-    if (ecx & (1<<25)) strcat(cpuid_info.features, "AES ");
-    if (ecx & (1<<28)) strcat(cpuid_info.features, "AVX ");
+    cpuid_info.featureFlags = 0;
+    cpuid_info.features = (char*) malloc(200*sizeof(char));
+    cpuid_info.features[0] = 0;
+    if (ecx & (1<<0))
+    {
+        strcat(cpuid_info.features, "SSE3 ");
+        cpuid_info.featureFlags |= (1<<SSE3);
+    }
+    if (ecx & (1<<3))
+    {
+        strcat(cpuid_info.features, "MONITOR ");
+        cpuid_info.featureFlags |= (1<<MONITOR);
+    }
+    if (ecx & (1<<5))
+    {
+        strcat(cpuid_info.features, "VMX ");
+        cpuid_info.featureFlags |= (1<<VMX);
+    }
+    if (ecx & (1<<7))
+    {
+        strcat(cpuid_info.features, "EIST ");
+        cpuid_info.featureFlags |= (1<<EIST);
+    }
+    if (ecx & (1<<8))
+    {
+        strcat(cpuid_info.features, "TM2 ");
+        cpuid_info.featureFlags |= (1<<TM2);
+    }
+    if (ecx & (1<<9))
+    {
+        strcat(cpuid_info.features, "SSSE3 ");
+        cpuid_info.featureFlags |= (1<<SSSE3);
+    }
+    if (ecx & (1<<12))
+    {
+        strcat(cpuid_info.features, "FMA ");
+        cpuid_info.featureFlags |= (1<<FMA);
+    }
+    if (ecx & (1<<19))
+    {
+        strcat(cpuid_info.features, "SSE4.1 ");
+        cpuid_info.featureFlags |= (1<<SSE41);
+    }
+    if (ecx & (1<<20))
+    {
+        strcat(cpuid_info.features, "SSE4.2 ");
+        cpuid_info.featureFlags |= (1<<SSE42);
+    }
+    if (ecx & (1<<25))
+    {
+        strcat(cpuid_info.features, "AES ");
+        cpuid_info.featureFlags |= (1<<AES);
+    }
+    if (ecx & (1<<28))
+    {
+        strcat(cpuid_info.features, "AVX ");
+        cpuid_info.featureFlags |= (1<<AVX);
+    }
+    if (ecx & (1<<30))
+    {
+        strcat(cpuid_info.features, "RDRAND ");
+        cpuid_info.featureFlags |= (1<<RDRAND);
+    }
+    if (edx & (1<<22))
+    {
+        strcat(cpuid_info.features, "ACPI ");
+        cpuid_info.featureFlags |= (1<<ACPI);
+    }
+    if (edx & (1<<23))
+    {
+        strcat(cpuid_info.features, "MMX ");
+        cpuid_info.featureFlags |= (1<<MMX);
+    }
+    if (edx & (1<<25))
+    {
+        strcat(cpuid_info.features, "SSE ");
+        cpuid_info.featureFlags |= (1<<SSE);
+    }
+    if (edx & (1<<26))
+    {
+        strcat(cpuid_info.features, "SSE2 ");
+        cpuid_info.featureFlags |= (1<<SSE2);
+    }
+    if (edx & (1<<29))
+    {
+        strcat(cpuid_info.features, "TM ");
+        cpuid_info.featureFlags |= (1<<TM);
+    }
+
+    eax = 0x80000001;
+    CPUID;
+    if (edx & (1<<27))
+    {
+        strcat(cpuid_info.features, "RDTSCP ");
+        cpuid_info.featureFlags |= (1<<RDTSCP);
+    }
 
     cpuid_info.perf_version   =  0;
-    if( cpuid_info.family == P6_FAMILY && 0x0A <= largest_function) 
+    if( cpuid_info.family == P6_FAMILY && 0x0A <= largest_function)
     {
         eax = 0x0A;
         CPUID;
@@ -468,37 +697,82 @@ cpuid_init (void)
         }
     }
 
-    cpuid_topology.numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
+    FILE *file;
+    char *filepath = TOSTRING(CFGFILE);
 
-    //    if (!cpuid_isInCpuset())
-    //    {
-    cpu_set_t cpuSet;
-    CPU_ZERO(&cpuSet);
-    sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
-    cpuid_initTopology();
-    cpuid_initCacheTopology();
+    if ((file = fopen(filepath, "rb")) != NULL) 
+    {
+        //printf("Read config from file\n");
+        initTopology(file);
+        fclose(file);
+    }
+    else
+    {
+        cpuid_topology.numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
+        if (recheck_numHWThreads() != cpuid_topology.numHWThreads)
+        {
+            cpuid_topology.numHWThreads = recheck_numHWThreads();
+        }
+        cpu_set_t cpuSet;
+        CPU_ZERO(&cpuSet);
+        sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
+        cpuid_initTopology();
+        cpuid_initCacheTopology();
 
-    /* restore affinity mask of process */
-    sched_setaffinity(0, sizeof(cpu_set_t), &cpuSet);
-    //    }
-    //    else
-    //    {
-    //        likwid_inCPUSet = 1;
-    //        printf("WARNING: You are running inside a restricted cpuset.\n");
-    //        printf("You can only use physical processor ids or use logical ids inside your cpuset.\n");
-    //    }
+        /* restore affinity mask of process */
+        sched_setaffinity(0, sizeof(cpu_set_t), &cpuSet);
+    }
 
-    return;
+    return EXIT_SUCCESS;
 }
 
+void cpuid_print (void)
+{
+    printf("\nSupported Intel processors:\n");
+    printf("\t%s\n",core_2a_str);
+    printf("\t%s\n",core_2b_str);
+    printf("\t%s\n",xeon_mp_string);
+    printf("\t%s\n",atom_45_str);
+    printf("\t%s\n",atom_32_str);
+    printf("\t%s\n",atom_22_str);
+    printf("\t%s\n",nehalem_bloom_str);
+    printf("\t%s\n",nehalem_lynn_str);
+    printf("\t%s\n",nehalem_west_str);
+    printf("\t%s (with Uncore support)\n",nehalem_ex_str);
+    printf("\t%s (with Uncore support)\n",westmere_ex_str);
+    printf("\t%s\n",sandybridge_str);
+    printf("\t%s (with Uncore support)\n",sandybridge_ep_str);
+    printf("\t%s\n",ivybridge_str);
+    printf("\t%s (with Uncore support)\n",ivybridge_ep_str);
+    printf("\t%s (with Uncore support)\n",haswell_str);
+    printf("\t%s (no Uncore support)\n",haswell_ex_str);
+    printf("\t%s\n",atom_silvermont_str);
+    printf("\t%s\n",atom_saltwell_str);
+    printf("\t%s\n\n",xeon_phi_string);
+
+    printf("Supported AMD processors:\n");
+    printf("\t%s\n",opteron_sc_str);
+    printf("\t%s\n",opteron_dc_e_str);
+    printf("\t%s\n",opteron_dc_f_str);
+    printf("\t%s\n",barcelona_str);
+    printf("\t%s\n",shanghai_str);
+    printf("\t%s\n",istanbul_str);
+    printf("\t%s\n",magnycours_str);
+    printf("\t%s\n",interlagos_str);
+    printf("\t%s\n\n",kabini_str);
+}
+
+
+
+
+
 #define freeStrings  \
-bdestroy(filename);  \
+    bdestroy(filename);  \
 bdestroy(grepString); \
 bdestroy(cpulist)
 
 
-int
-cpuid_isInCpuset(void)
+int cpuid_isInCpuset(void)
 {
     int pos = 0;
     bstring grepString = bformat("Cpus_allowed_list:");
@@ -538,12 +812,10 @@ cpuid_isInCpuset(void)
     }
 }
 
-void
-cpuid_initTopology(void)
+void cpuid_initTopology(void)
 {
     uint32_t apicId;
     uint32_t bitField;
-    uint32_t i;
     int level;
     int prevOffset = 0;
     int currOffset = 0;
@@ -563,7 +835,7 @@ cpuid_initTopology(void)
         ecx = 0;
         CPUID;
 
-        if (ebx) 
+        if (ebx)
         {
             hasBLeaf = 1;
         }
@@ -572,14 +844,9 @@ cpuid_initTopology(void)
     hwThreadPool = (HWThread*) malloc(cpuid_topology.numHWThreads * sizeof(HWThread));
     tree_init(&cpuid_topology.topologyTree, 0);
 
-    if ( likwid_inCPUSet )
+    if (hasBLeaf)
     {
-
-
-    }
-    else if (hasBLeaf)
-    {
-        for (i=0; i < cpuid_topology.numHWThreads; i++)
+        for (uint32_t i=0; i < cpuid_topology.numHWThreads; i++)
         {
 
             CPU_ZERO(&set);
@@ -627,7 +894,7 @@ cpuid_initTopology(void)
     }
     else
     {
-        switch ( cpuid_info.family ) 
+        switch ( cpuid_info.family )
         {
 
             case MIC_FAMILY:
@@ -645,7 +912,7 @@ cpuid_initTopology(void)
 
                 maxNumLogicalProcsPerCore = maxNumLogicalProcs/maxNumCores;
 
-                for (i=0; i<  cpuid_topology.numHWThreads; i++)
+                for (uint32_t i=0; i<  cpuid_topology.numHWThreads; i++)
                 {
                     CPU_ZERO(&set);
                     CPU_SET(i,&set);
@@ -691,7 +958,7 @@ cpuid_initTopology(void)
 
                 maxNumCores =  extractBitField(ecx,8,0)+1;
 
-                for (i=0; i<  cpuid_topology.numHWThreads; i++)
+                for (uint32_t i=0; i<  cpuid_topology.numHWThreads; i++)
                 {
                     CPU_ZERO(&set);
                     CPU_SET(i,&set);
@@ -724,6 +991,8 @@ cpuid_initTopology(void)
                 }
                 break;
 
+            case K16_FAMILY:
+
             case K15_FAMILY:
 
             case K10_FAMILY:
@@ -745,7 +1014,7 @@ cpuid_initTopology(void)
                 maxNumCores = extractBitField(ecx,8,0)+1;
 
 
-                for (i=0; i<  cpuid_topology.numHWThreads; i++)
+                for (uint32_t i=0; i<  cpuid_topology.numHWThreads; i++)
                 {
                     CPU_ZERO(&set);
                     CPU_SET(i,&set);
@@ -769,7 +1038,7 @@ cpuid_initTopology(void)
         }
     }
 
-    for (i=0; i<  cpuid_topology.numHWThreads; i++)
+    for (uint32_t i=0; i<  cpuid_topology.numHWThreads; i++)
     {
         /* Add node to Topology tree */
         if (!tree_nodeExists(cpuid_topology.topologyTree,
@@ -805,8 +1074,7 @@ cpuid_initTopology(void)
     cpuid_topology.numThreadsPerCore = tree_countChildren(currentNode);
 }
 
-void 
-cpuid_initCacheTopology()
+void cpuid_initCacheTopology()
 {
     int maxNumLevels=0;
     int id=0;
@@ -932,6 +1200,8 @@ cpuid_initCacheTopology()
 
             break;
 
+        case K16_FAMILY:
+
         case K15_FAMILY:
 
             maxNumLevels = 0;
