@@ -5,13 +5,13 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,7 +42,7 @@ extern int  affinity_threadGetProcessorId();
 extern void  affinity_pinProcess(int processorId);
 extern void  affinity_pinThread(int processorId);
 extern const AffinityDomain* affinity_getDomain(bstring domain);
-extern void affinity_printDomains();
+extern void affinity_printDomains(FILE* OUTSTREAM);
 
 #endif /*AFFINITY_H*/
 
