@@ -5,13 +5,13 @@
  *
  *      Description:  Type Definitions for affinity Module
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,8 @@
 typedef struct {
     bstring tag;
     uint32_t numberOfProcessors;
-    int*  processorList;
+    uint32_t numberOfCores;
+    int* processorList;
 } AffinityDomain;
 
 
