@@ -5,13 +5,13 @@
  *
  *      Description:  Header File daemon Module. 
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -35,7 +35,7 @@
 #include <time.h>
 
 extern void daemon_init();
-extern void daemon_start(struct timespec interval);
+extern void daemon_start(bstring str, struct timespec interval);
 extern void daemon_stop(int sig);
 extern void daemon_interrupt(int sig);
 
