@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -36,11 +36,10 @@
 typedef struct LikwidThreadResults{
     bstring  label;
     double time;
-    CyclesData startTime;
+    TimerData startTime;
     uint32_t count;
     double StartPMcounters[NUM_PMC];
     double PMcounters[NUM_PMC];
-    struct LikwidThreadResults* next;
 } LikwidThreadResults;
 
 typedef struct {
