@@ -5,13 +5,13 @@
  *
  *      Description:  Header File memsweep Module. 
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,9 +34,9 @@
 #include <types.h>
 
 extern void memsweep_setMemoryFraction(uint64_t fraction);
-extern void memsweep_node(void);
-extern void memsweep_domain(int domainId);
-extern void memsweep_threadGroup(int* processorList, int numberOfProcessors);
+extern void memsweep_node(FILE* OUTSTREAM);
+extern void memsweep_domain(FILE* OUTSTREAM, int domainId);
+extern void memsweep_threadGroup(FILE* OUTSTREAM, int* processorList, int numberOfProcessors);
 
 #endif /* MEMSWEEP_H */
 
