@@ -3,15 +3,15 @@
  *
  *      Filename:  perfmon_atom.h
  *
- *      Description:  Atom specific subroutines
+ *      Description:  Header file of perfmon module for Atom
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -28,14 +28,9 @@
  * =======================================================================================
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-
 #include <perfmon_atom_events.h>
 #include <perfmon_atom_groups.h>
 
-
 static int perfmon_numGroupsAtom = NUM_GROUPS_ATOM;
 static int perfmon_numArchEventsAtom = NUM_ARCH_EVENTS_ATOM;
 
-
