@@ -3,15 +3,15 @@
  *
  *      Filename:  perfmon_core2.h
  *
- *      Description:  Core 2 specific subroutines
+ *      Description:  Header file of perfmon module for Core 2
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,44 +30,17 @@
 
 #include <perfmon_core2_events.h>
 #include <perfmon_core2_groups.h>
-
-#define NUM_COUNTERS_CORE2 4
-#define NUM_SETS_CORE2 8
+#include <perfmon_core2_counters.h>
 
 static int perfmon_numCountersCore2 = NUM_COUNTERS_CORE2;
 static int perfmon_numGroupsCore2 = NUM_GROUPS_CORE2;
 static int perfmon_numArchEventsCore2 = NUM_ARCH_EVENTS_CORE2;
 
-static PerfmonCounterMap core2_counter_map[NUM_COUNTERS_CORE2] = {
-    {"FIXC0",PMC0},
-    {"FIXC1",PMC1},
-    {"PMC0",PMC2},
-    {"PMC1",PMC3}
-};
-
-
-void 
-perfmon_init_core2(PerfmonThread *thread)
+void perfmon_init_core2(PerfmonThread *thread)
 {
     uint64_t flags = 0x0ULL;
     int cpu_id = thread->processorId;
 
-    /* Fixed Counters: instructions retired, cycles unhalted core */
-    thread->counters[PMC0].configRegister = MSR_PERF_FIXED_CTR_CTRL;
-    thread->counters[PMC0].counterRegister = MSR_PERF_FIXED_CTR0;
-    thread->counters[PMC0].type = FIXED;
-    thread->counters[PMC1].configRegister = MSR_PERF_FIXED_CTR_CTRL;
-    thread->counters[PMC1].counterRegister = MSR_PERF_FIXED_CTR1;
-    thread->counters[PMC1].type = FIXED;
-
-    /* PMC Counters: 2 40bit wide */
-    thread->counters[PMC2].configRegister = MSR_PERFEVTSEL0;
-    thread->counters[PMC2].counterRegister = MSR_PMC0;
-    thread->counters[PMC2].type = PMC;
-    thread->counters[PMC3].configRegister = MSR_PERFEVTSEL1;
-    thread->counters[PMC3].counterRegister = MSR_PMC1;
-    thread->counters[PMC3].type = PMC;
-
     /* Initialize registers */
     msr_write(cpu_id, MSR_PERFEVTSEL0, 0x0ULL);
     msr_write(cpu_id, MSR_PERFEVTSEL1, 0x0ULL);
@@ -92,21 +65,20 @@ perfmon_init_core2(PerfmonThread *thread)
 }
 
 
-void
-perfmon_setupCounterThread_core2(int thread_id,
+void perfmon_setupCounterThread_core2(
+        int thread_id,
         PerfmonEvent* event,
         PerfmonCounterIndex index)
 {
     uint64_t flags;
-    uint64_t reg = perfmon_threadData[thread_id].counters[index].configRegister;
+    uint64_t reg = core2_counter_map[index].configRegister;
     int cpu_id = perfmon_threadData[thread_id].processorId;
+    uint64_t fixed_flags = msr_read(cpu_id, MSR_PERF_FIXED_CTR_CTRL);
+    perfmon_threadData[thread_id].counters[index].init = TRUE;
 
-    if ( perfmon_threadData[thread_id].counters[index].type == PMC )
+    if ( core2_counter_map[index].type == PMC )
     {
-
-        perfmon_threadData[thread_id].counters[index].init = TRUE;
-        flags = msr_read(cpu_id,reg);
-        flags &= ~(0xFFFFU); 
+        flags = (1<<16)|(1<<19)|(1<<22);
 
         /* Intel with standard 8 bit event mask: [7:0] */
         flags |= (event->umask<<8) + event->eventId;
@@ -127,31 +99,31 @@ perfmon_setupCounterThread_core2(int thread_id,
                     LLU_CAST flags);
         }
     }
-    else if (perfmon_threadData[thread_id].counters[index].type == FIXED)
+    else if (core2_counter_map[index].type == FIXED)
     {
-        perfmon_threadData[thread_id].counters[index].init = TRUE;
+        fixed_flags |= (0x2 << (index*4));
+        msr_write(cpu_id, MSR_PERF_FIXED_CTR_CTRL, fixed_flags);
     }
 }
 
-void
-perfmon_startCountersThread_core2(int thread_id)
+void perfmon_startCountersThread_core2(int thread_id)
 {
     uint64_t flags = 0ULL;
     int cpu_id = perfmon_threadData[thread_id].processorId;
 
     msr_write(cpu_id, MSR_PERF_GLOBAL_CTRL, 0x0ULL);
 
-    for ( int i=0; i<NUM_COUNTERS_CORE2; i++ ) 
+    for ( int i=0; i<NUM_COUNTERS_CORE2; i++ )
     {
-        if (perfmon_threadData[thread_id].counters[i].init == TRUE) 
+        if (perfmon_threadData[thread_id].counters[i].init == TRUE)
         {
-            msr_write(cpu_id, perfmon_threadData[thread_id].counters[i].counterRegister , 0x0ULL);
+            msr_write(cpu_id, core2_counter_map[i].counterRegister , 0x0ULL);
 
-            if (perfmon_threadData[thread_id].counters[i].type == PMC)
+            if (core2_counter_map[i].type == PMC)
             {
                 flags |= (1<<(i-2));  /* enable counter */
             }
-            else if (perfmon_threadData[thread_id].counters[i].type == FIXED)
+            else if (core2_counter_map[i].type == FIXED)
             {
                 flags |= (1ULL<<(i+32));  /* enable fixed counter */
             }
@@ -168,8 +140,7 @@ perfmon_startCountersThread_core2(int thread_id)
     msr_write(cpu_id, MSR_PERF_GLOBAL_OVF_CTRL, 0x300000003ULL);
 }
 
-void 
-perfmon_stopCountersThread_core2(int thread_id)
+void perfmon_stopCountersThread_core2(int thread_id)
 {
     uint64_t flags;
     int cpu_id = perfmon_threadData[thread_id].processorId;
@@ -178,35 +149,34 @@ perfmon_stopCountersThread_core2(int thread_id)
     msr_write(cpu_id, MSR_PERF_GLOBAL_CTRL, 0x0ULL);
 
     /* read out counter results */
-    for ( int i=0; i<NUM_COUNTERS_CORE2; i++) 
+    for ( int i=0; i<NUM_COUNTERS_CORE2; i++)
     {
-        if ( perfmon_threadData[thread_id].counters[i].init == TRUE ) 
+        if ( perfmon_threadData[thread_id].counters[i].init == TRUE )
         {
             perfmon_threadData[thread_id].counters[i].counterData =
-                msr_read(cpu_id, perfmon_threadData[thread_id].counters[i].counterRegister);
+                msr_read(cpu_id, core2_counter_map[i].counterRegister);
         }
     }
 
     /* check overflow status */
     flags = msr_read(cpu_id,MSR_PERF_GLOBAL_STATUS);
-    if ( (flags & 0x3) || (flags & (0x3ULL<<32)) ) 
+    if ( (flags & 0x3) || (flags & (0x3ULL<<32)) )
     {
         printf ("Overflow occured \n");
         printf ("Status: 0x%llX \n", LLU_CAST flags);
     }
 }
 
-void 
-perfmon_readCountersThread_core2(int thread_id)
+void perfmon_readCountersThread_core2(int thread_id)
 {
     int cpu_id = perfmon_threadData[thread_id].processorId;
 
-    for ( int i=0; i<NUM_COUNTERS_CORE2; i++ ) 
+    for ( int i=0; i<NUM_COUNTERS_CORE2; i++ )
     {
-        if ( perfmon_threadData[thread_id].counters[i].init == TRUE ) 
+        if ( perfmon_threadData[thread_id].counters[i].init == TRUE )
         {
             perfmon_threadData[thread_id].counters[i].counterData =
-                msr_read(cpu_id, perfmon_threadData[thread_id].counters[i].counterRegister);
+                msr_read(cpu_id, core2_counter_map[i].counterRegister);
         }
     }
 }
