@@ -7,13 +7,13 @@
  *                    Configures and reads out performance counters
  *                    on x86 based architectures. Supports multi threading.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,12 +47,23 @@ typedef enum {
     PMC25, PMC26, PMC27, PMC28, PMC29, PMC30,
     PMC31, PMC32, PMC33, PMC34, PMC35, PMC36,
     PMC37, PMC38, PMC39, PMC40, PMC41, PMC42,
-    PMC43, PMC44, PMC45, PMC46, PMC47,
+    PMC43, PMC44, PMC45, PMC46, PMC47, PMC48,
+    PMC49, PMC50, PMC51, PMC52, PMC53, PMC54,
+    PMC55, PMC56, PMC57, PMC58, PMC59, PMC60,
+    PMC61, PMC62, PMC63, PMC64, PMC65, PMC66,
+    PMC67, PMC68, PMC69, PMC70, PMC71, PMC72,
+    PMC73, PMC74, PMC75, PMC76, PMC77, PMC78,
+    PMC79, PMC80, PMC81, PMC82, PMC83, PMC84,
+    PMC85, PMC86, PMC87, PMC88, PMC89, PMC90,
+    PMC91, PMC92, PMC93, PMC94, PMC95, PMC96,
+    PMC97, PMC98, PMC99, PMC100, PMC101, PMC102,
+    PMC103, PMC104, PMC105, PMC106, PMC107, PMC108,
     NUM_PMC} PerfmonCounterIndex;
 
 typedef enum {
     PMC = 0,
     FIXED,
+    THERMAL,
     UNCORE,
     MBOX0,
     MBOX1,
@@ -67,21 +78,44 @@ typedef enum {
     SBOX0,
     SBOX1,
     SBOX2,
-    CBOX,
+    CBOX0,
+    CBOX1,
+    CBOX2,
+    CBOX3,
+    CBOX4,
+    CBOX5,
+    CBOX6,
+    CBOX7,
+    CBOX8,
+    CBOX9,
+    CBOX10,
+    CBOX11,
+    CBOX12,
+    CBOX13,
+    CBOX14,
     PBOX,
     POWER,
+    UBOX,
     NUM_UNITS} PerfmonType;
 
 typedef struct {
     char* key;
     PerfmonCounterIndex index;
+    PerfmonType type;
+    uint64_t configRegister;
+    uint64_t counterRegister;
+    uint64_t counterRegister2;
+    PciDeviceIndex device;
 } PerfmonCounterMap;
 
 typedef struct {
-    char* key;
+    const char* key;
     PerfmonGroup index;
-    char* info;
-    char* config;
+    int isUncore;
+    const char* info;
+    const char* config;
+    int derivedCounters;
+    const char ** derivedCounterNames;
 } PerfmonGroupMap;
 
 typedef struct {
@@ -91,19 +125,15 @@ typedef struct {
 
 /* only used in westmereEX at the moment */
 typedef struct {
-    uint32_t  ctrlRegister;
-    uint32_t  statusRegister;
-    uint32_t  ovflRegister;
+    uint32_t ctrlRegister;
+    uint32_t statusRegister;
+    uint32_t ovflRegister;
 } PerfmonUnit;
 
 typedef struct {
-    PerfmonType  type;
-    int       init;
-    int       id;
-    uint64_t  configRegister;
-    uint64_t  counterRegister;
-    uint64_t  counterRegister2;
-    uint64_t  counterData;
+    int init;
+    int id;  /* TODO id is only used for EX type processors */
+    double counterData;
 } PerfmonCounter;
 
 typedef struct {
@@ -112,8 +142,8 @@ typedef struct {
 } PerfmonThread;
 
 typedef struct {
-    const char*    name;
-    const char*    limit;
+    const char* name;
+    const char* limit;
     uint16_t eventId;
     uint8_t umask;
     uint8_t cfgBits;
