@@ -4,16 +4,14 @@
  *      Filename:  perfmon_westmere.h
  *
  *      Description:  Header File of perfmon module for Westmere.
- *                    Configures and reads out performance counters
- *                    on x86 based architectures. Supports multi threading.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,14 +28,6 @@
  * =======================================================================================
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-
-#include <cpuid.h>
-#include <tree.h>
-#include <bstrlib.h>
-#include <types.h>
-#include <registers.h>
 #include <perfmon_westmere_events.h>
 #include <perfmon_westmere_groups.h>
 
