@@ -5,13 +5,13 @@
  *
  *      Description:  Register Defines for the perfmon module
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -61,13 +61,14 @@
 #define MSR_PERF_GLOBAL_OVF_CTRL  0x390
 #define MSR_PEBS_ENABLE           0x3F1
 /* Perfmon V3 */
-#define MSR_OFFCORE_RSP0              0x1A6
+#define MSR_OFFCORE_RESP0              0x1A6
+#define MSR_OFFCORE_RESP1              0x1A7
 #define MSR_UNCORE_PERF_GLOBAL_CTRL       0x391
 #define MSR_UNCORE_PERF_GLOBAL_STATUS     0x392
 #define MSR_UNCORE_PERF_GLOBAL_OVF_CTRL   0x393
 #define MSR_UNCORE_FIXED_CTR0             0x394  /* Uncore clock cycles */
 #define MSR_UNCORE_FIXED_CTR_CTRL         0x395 /*FIXME: Is this correct? */
-#define MSR_UNCORE_ADDR_OPCODE_MATCH      0x396 
+#define MSR_UNCORE_ADDR_OPCODE_MATCH      0x396
 #define MSR_UNCORE_PERFEVTSEL0         0x3C0
 #define MSR_UNCORE_PERFEVTSEL1         0x3C1
 #define MSR_UNCORE_PERFEVTSEL2         0x3C2
@@ -84,7 +85,36 @@
 #define MSR_UNCORE_PMC5                0x3B5
 #define MSR_UNCORE_PMC6                0x3B6
 #define MSR_UNCORE_PMC7                0x3B7
-
+/*
+ * Perfmon V4 (starting with Haswell, according to
+ * Intel software developers guide also for SandyBridge,
+ * IvyBridge not mentioned in this section)
+ */
+#define MSR_UNC_PERF_GLOBAL_CTRL       MSR_UNCORE_PERF_GLOBAL_CTRL
+#define MSR_UNC_PERF_GLOBAL_STATUS     MSR_UNCORE_PERF_GLOBAL_STATUS
+#define MSR_UNC_PERF_FIXED_CTRL        MSR_UNCORE_FIXED_CTR0
+#define MSR_UNC_PERF_FIXED_CTR         MSR_UNCORE_FIXED_CTR_CTRL
+#define MSR_UNC_ARB_PERFEVTSEL0        MSR_UNCORE_PMC2
+#define MSR_UNC_ARB_PERFEVTSEL1        MSR_UNCORE_PMC3
+#define MSR_UNC_ARB_CTR0               MSR_UNCORE_PMC0
+#define MSR_UNC_ARB_CTR1               MSR_UNCORE_PMC1
+#define MSR_UNC_CBO_CONFIG             0x396
+#define MSR_UNC_CBO_0_PERFEVTSEL0      0x700
+#define MSR_UNC_CBO_0_PERFEVTSEL1      0x701
+#define MSR_UNC_CBO_0_CTR0             0x706
+#define MSR_UNC_CBO_0_CTR1             0x707
+#define MSR_UNC_CBO_1_PERFEVTSEL0      0x710
+#define MSR_UNC_CBO_1_PERFEVTSEL1      0x711
+#define MSR_UNC_CBO_1_CTR0             0x716
+#define MSR_UNC_CBO_1_CTR1             0x717
+#define MSR_UNC_CBO_2_PERFEVTSEL0      0x720
+#define MSR_UNC_CBO_2_PERFEVTSEL1      0x721
+#define MSR_UNC_CBO_2_CTR0             0x726
+#define MSR_UNC_CBO_2_CTR1             0x727
+#define MSR_UNC_CBO_3_PERFEVTSEL0      0x730
+#define MSR_UNC_CBO_3_PERFEVTSEL1      0x731
+#define MSR_UNC_CBO_3_CTR0             0x736
+#define MSR_UNC_CBO_3_CTR1             0x737
 /* Xeon Phi */
 #define MSR_MIC_TSC                   0x010
 #define MSR_MIC_PERFEVTSEL0           0x028
@@ -92,14 +122,14 @@
 #define MSR_MIC_PMC0                  0x020
 #define MSR_MIC_PMC1                  0x021
 #define MSR_MIC_SPFLT_CONTROL         0x02C
-#define MSR_MIC_PERF_GLOBAL_STATUS    0x02D 
-#define MSR_MIC_PERF_GLOBAL_OVF_CTRL  0x02E 
-#define MSR_MIC_PERF_GLOBAL_CTRL      0x02F 
+#define MSR_MIC_PERF_GLOBAL_STATUS    0x02D
+#define MSR_MIC_PERF_GLOBAL_OVF_CTRL  0x02E
+#define MSR_MIC_PERF_GLOBAL_CTRL      0x02F
 
 
-/* Sandy Bridge type uncore 
+/* Core v1/v2 type uncore
  * Naming following Intel Uncore Performance Monitoring Guide
- * Ref. Nr. 327043-001
+ * Ref. Nr. 327043-001 and 329468-001
  * */
 
 /* CBo Performance Monitoring */
@@ -112,6 +142,7 @@
 #define MSR_UNC_C0_PMON_CTL2           0xD12
 #define MSR_UNC_C0_PMON_CTL3           0xD13
 #define MSR_UNC_C0_PMON_BOX_FILTER     0xD14
+#define MSR_UNC_C0_PMON_BOX_FILTER1    0xD1A
 #define MSR_UNC_C0_PMON_BOX_CTL        0xD04
 
 #define MSR_UNC_C1_PMON_CTR0           0xD36
@@ -123,6 +154,7 @@
 #define MSR_UNC_C1_PMON_CTL2           0xD32
 #define MSR_UNC_C1_PMON_CTL3           0xD33
 #define MSR_UNC_C1_PMON_BOX_FILTER     0xD34
+#define MSR_UNC_C1_PMON_BOX_FILTER1    0xD3A
 #define MSR_UNC_C1_PMON_BOX_CTL        0xD24
 
 #define MSR_UNC_C2_PMON_CTR0           0xD56
@@ -134,6 +166,7 @@
 #define MSR_UNC_C2_PMON_CTL2           0xD52
 #define MSR_UNC_C2_PMON_CTL3           0xD53
 #define MSR_UNC_C2_PMON_BOX_FILTER     0xD54
+#define MSR_UNC_C2_PMON_BOX_FILTER1    0xD5A
 #define MSR_UNC_C2_PMON_BOX_CTL        0xD44
 
 #define MSR_UNC_C3_PMON_CTR0           0xD76
@@ -145,6 +178,7 @@
 #define MSR_UNC_C3_PMON_CTL2           0xD72
 #define MSR_UNC_C3_PMON_CTL3           0xD73
 #define MSR_UNC_C3_PMON_BOX_FILTER     0xD74
+#define MSR_UNC_C3_PMON_BOX_FILTER1    0xD7A
 #define MSR_UNC_C3_PMON_BOX_CTL        0xD64
 
 #define MSR_UNC_C4_PMON_CTR0           0xD96
@@ -156,6 +190,7 @@
 #define MSR_UNC_C4_PMON_CTL2           0xD92
 #define MSR_UNC_C4_PMON_CTL3           0xD93
 #define MSR_UNC_C4_PMON_BOX_FILTER     0xD94
+#define MSR_UNC_C4_PMON_BOX_FILTER1    0xD9A
 #define MSR_UNC_C4_PMON_BOX_CTL        0xD84
 
 #define MSR_UNC_C5_PMON_CTR0           0xDB6
@@ -167,6 +202,7 @@
 #define MSR_UNC_C5_PMON_CTL2           0xDB2
 #define MSR_UNC_C5_PMON_CTL3           0xDB3
 #define MSR_UNC_C5_PMON_BOX_FILTER     0xDB4
+#define MSR_UNC_C5_PMON_BOX_FILTER1    0xDBA
 #define MSR_UNC_C5_PMON_BOX_CTL        0xDA4
 
 #define MSR_UNC_C6_PMON_CTR0           0xDD6
@@ -178,6 +214,7 @@
 #define MSR_UNC_C6_PMON_CTL2           0xDD2
 #define MSR_UNC_C6_PMON_CTL3           0xDD3
 #define MSR_UNC_C6_PMON_BOX_FILTER     0xDD4
+#define MSR_UNC_C6_PMON_BOX_FILTER1    0xDDA
 #define MSR_UNC_C6_PMON_BOX_CTL        0xDC4
 
 #define MSR_UNC_C7_PMON_CTR0           0xDF6
@@ -189,8 +226,93 @@
 #define MSR_UNC_C7_PMON_CTL2           0xDF2
 #define MSR_UNC_C7_PMON_CTL3           0xDF3
 #define MSR_UNC_C7_PMON_BOX_FILTER     0xDF4
+#define MSR_UNC_C7_PMON_BOX_FILTER1    0xDFA
 #define MSR_UNC_C7_PMON_BOX_CTL        0xDE4
 
+#define MSR_UNC_C8_PMON_CTR0           0xE16
+#define MSR_UNC_C8_PMON_CTR1           0xE17
+#define MSR_UNC_C8_PMON_CTR2           0xE18
+#define MSR_UNC_C8_PMON_CTR3           0xE19
+#define MSR_UNC_C8_PMON_CTL0           0xE10
+#define MSR_UNC_C8_PMON_CTL1           0xE11
+#define MSR_UNC_C8_PMON_CTL2           0xE12
+#define MSR_UNC_C8_PMON_CTL3           0xE13
+#define MSR_UNC_C8_PMON_BOX_FILTER     0xE14
+#define MSR_UNC_C8_PMON_BOX_FILTER1    0xE1A
+#define MSR_UNC_C8_PMON_BOX_CTL        0xE04
+
+#define MSR_UNC_C9_PMON_CTR0           0xE36
+#define MSR_UNC_C9_PMON_CTR1           0xE37
+#define MSR_UNC_C9_PMON_CTR2           0xE38
+#define MSR_UNC_C9_PMON_CTR3           0xE39
+#define MSR_UNC_C9_PMON_CTL0           0xE30
+#define MSR_UNC_C9_PMON_CTL1           0xE31
+#define MSR_UNC_C9_PMON_CTL2           0xE32
+#define MSR_UNC_C9_PMON_CTL3           0xE33
+#define MSR_UNC_C9_PMON_BOX_FILTER     0xE34
+#define MSR_UNC_C9_PMON_BOX_FILTER1    0xE3A
+#define MSR_UNC_C9_PMON_BOX_CTL        0xE24
+
+#define MSR_UNC_C10_PMON_CTR0           0xE56
+#define MSR_UNC_C10_PMON_CTR1           0xE57
+#define MSR_UNC_C10_PMON_CTR2           0xE58
+#define MSR_UNC_C10_PMON_CTR3           0xE59
+#define MSR_UNC_C10_PMON_CTL0           0xE50
+#define MSR_UNC_C10_PMON_CTL1           0xE51
+#define MSR_UNC_C10_PMON_CTL2           0xE52
+#define MSR_UNC_C10_PMON_CTL3           0xE53
+#define MSR_UNC_C10_PMON_BOX_FILTER     0xE54
+#define MSR_UNC_C10_PMON_BOX_FILTER1    0xE5A
+#define MSR_UNC_C10_PMON_BOX_CTL        0xE44
+
+#define MSR_UNC_C11_PMON_CTR0           0xE76
+#define MSR_UNC_C11_PMON_CTR1           0xE77
+#define MSR_UNC_C11_PMON_CTR2           0xE78
+#define MSR_UNC_C11_PMON_CTR3           0xE79
+#define MSR_UNC_C11_PMON_CTL0           0xE70
+#define MSR_UNC_C11_PMON_CTL1           0xE71
+#define MSR_UNC_C11_PMON_CTL2           0xE72
+#define MSR_UNC_C11_PMON_CTL3           0xE73
+#define MSR_UNC_C11_PMON_BOX_FILTER     0xE74
+#define MSR_UNC_C11_PMON_BOX_FILTER1    0xE7A
+#define MSR_UNC_C11_PMON_BOX_CTL        0xE64
+
+#define MSR_UNC_C12_PMON_CTR0           0xE96
+#define MSR_UNC_C12_PMON_CTR1           0xE97
+#define MSR_UNC_C12_PMON_CTR2           0xE98
+#define MSR_UNC_C12_PMON_CTR3           0xE99
+#define MSR_UNC_C12_PMON_CTL0           0xE90
+#define MSR_UNC_C12_PMON_CTL1           0xE91
+#define MSR_UNC_C12_PMON_CTL2           0xE92
+#define MSR_UNC_C12_PMON_CTL3           0xE93
+#define MSR_UNC_C12_PMON_BOX_FILTER     0xE94
+#define MSR_UNC_C12_PMON_BOX_FILTER1    0xE9A
+#define MSR_UNC_C12_PMON_BOX_CTL        0xE84
+
+#define MSR_UNC_C13_PMON_CTR0           0xEB6
+#define MSR_UNC_C13_PMON_CTR1           0xEB7
+#define MSR_UNC_C13_PMON_CTR2           0xEB8
+#define MSR_UNC_C13_PMON_CTR3           0xEB9
+#define MSR_UNC_C13_PMON_CTL0           0xEB0
+#define MSR_UNC_C13_PMON_CTL1           0xEB1
+#define MSR_UNC_C13_PMON_CTL2           0xEB2
+#define MSR_UNC_C13_PMON_CTL3           0xEB3
+#define MSR_UNC_C13_PMON_BOX_FILTER     0xEB4
+#define MSR_UNC_C13_PMON_BOX_FILTER1    0xEBA
+#define MSR_UNC_C13_PMON_BOX_CTL        0xEA4
+
+#define MSR_UNC_C14_PMON_CTR0           0xED6
+#define MSR_UNC_C14_PMON_CTR1           0xED7
+#define MSR_UNC_C14_PMON_CTR2           0xED8
+#define MSR_UNC_C14_PMON_CTR3           0xED9
+#define MSR_UNC_C14_PMON_CTL0           0xED0
+#define MSR_UNC_C14_PMON_CTL1           0xED1
+#define MSR_UNC_C14_PMON_CTL2           0xED2
+#define MSR_UNC_C14_PMON_CTL3           0xED3
+#define MSR_UNC_C14_PMON_BOX_FILTER     0xED4
+#define MSR_UNC_C14_PMON_BOX_FILTER1    0xEDA
+#define MSR_UNC_C14_PMON_BOX_CTL        0xEC4
+
 /* PCU (Power Control) Performance Monitoring */
 
 #define MSR_UNC_PCU_PMON_CTR0           0xC36
@@ -212,6 +334,10 @@
 #define MSR_UNC_U_PMON_CTL1             0xC11
 #define MSR_UNC_U_UCLK_FIXED_CTR        0xC09
 #define MSR_UNC_U_UCLK_FIXED_CTL        0xC08
+#define MSR_UNC_U_PMON_BOX_STATUS       0xC15
+#define MSR_UNC_U_PMON_GLOBAL_STATUS    0xC01
+#define MSR_UNC_U_PMON_GLOBAL_CTL       0xC00
+#define MSR_UNC_U_PMON_GLOBAL_CONFIG    0xC06
 
 /* HA Box Performance Monitoring */
 
@@ -235,6 +361,7 @@
 /* iMC Box Performance Monitoring */
 
 #define PCI_UNC_MC_PMON_BOX_CTL         0xF4
+#define PCI_UNC_MC_PMON_BOX_STATUS      0xF8
 #define PCI_UNC_MC_PMON_FIXED_CTL       0xF0
 #define PCI_UNC_MC_PMON_CTL_0           0xD8
 #define PCI_UNC_MC_PMON_CTL_1           0xDC
@@ -548,6 +675,38 @@
 #define MSR_C7_PMON_CTR4                0xDF9
 #define MSR_C7_PMON_EVNT_SEL5           0xDFA
 #define MSR_C7_PMON_CTR5                0xDFB
+/* C box 8 - Coherence Engine core 8 */
+#define MSR_C8_PMON_BOX_CTRL            0xF40
+#define MSR_C8_PMON_BOX_STATUS          0xF41
+#define MSR_C8_PMON_BOX_OVF_CTRL        0xF42
+#define MSR_C8_PMON_EVNT_SEL0           0xF50
+#define MSR_C8_PMON_CTR0                0xF51
+#define MSR_C8_PMON_EVNT_SEL1           0xF52
+#define MSR_C8_PMON_CTR1                0xF53
+#define MSR_C8_PMON_EVNT_SEL2           0xF54
+#define MSR_C8_PMON_CTR2                0xF55
+#define MSR_C8_PMON_EVNT_SEL3           0xF56
+#define MSR_C8_PMON_CTR3                0xF57
+#define MSR_C8_PMON_EVNT_SEL4           0xF58
+#define MSR_C8_PMON_CTR4                0xF59
+#define MSR_C8_PMON_EVNT_SEL5           0xF5A
+#define MSR_C8_PMON_CTR5                0xF5B
+/* C box 9 - Coherence Engine core 9 */
+#define MSR_C9_PMON_BOX_CTRL            0xFC0
+#define MSR_C9_PMON_BOX_STATUS          0xFC1
+#define MSR_C9_PMON_BOX_OVF_CTRL        0xFC2
+#define MSR_C9_PMON_EVNT_SEL0           0xFD0
+#define MSR_C9_PMON_CTR0                0xFD1
+#define MSR_C9_PMON_EVNT_SEL1           0xFD2
+#define MSR_C9_PMON_CTR1                0xFD3
+#define MSR_C9_PMON_EVNT_SEL2           0xFD4
+#define MSR_C9_PMON_CTR2                0xFD5
+#define MSR_C9_PMON_EVNT_SEL3           0xFD6
+#define MSR_C9_PMON_CTR3                0xFD7
+#define MSR_C9_PMON_EVNT_SEL4           0xFD8
+#define MSR_C9_PMON_CTR4                0xFD9
+#define MSR_C9_PMON_EVNT_SEL5           0xFDA
+#define MSR_C9_PMON_CTR5                0xFDB
 /* R box 0 - Router 0 */
 #define MSR_R0_PMON_BOX_CTRL            0xE00
 #define MSR_R0_PMON_BOX_STATUS          0xE01
@@ -645,12 +804,19 @@
 #define MSR_DRAM_PERF_STATUS            0x61B
 #define MSR_DRAM_POWER_INFO             0x61C
 
+/* TM/TM2 interface */
+#define IA32_THERM_STATUS               0x19C
+#define IA32_PACKAGE_THERM_STATUS       0x1B1
+#define MSR_TEMPERATURE_TARGET          0x1A2
+
 /* Turbo Boost Interface */
 #define MSR_IA32_MISC_ENABLE            0x1A0
 #define MSR_PLATFORM_INFO               0x0CE
 #define MSR_TURBO_POWER_CURRENT_LIMIT   0x1AC
 #define MSR_TURBO_RATIO_LIMIT           0x1AD
 
+/* Intel Silvermont's RAPL registers */
+#define MSR_PKG_POWER_INFO_SILVERMONT   0x66E
 /*
  * AMD
  */
@@ -690,5 +856,24 @@
 #define MSR_AMD15_NB_PMC2               0xC0010245
 #define MSR_AMD15_NB_PMC3               0xC0010247
 
+/* AMD 0x16 */
+#define MSR_AMD16_PERFEVTSEL0           0xC0010000
+#define MSR_AMD16_PERFEVTSEL1           0xC0010001
+#define MSR_AMD16_PERFEVTSEL2           0xC0010002
+#define MSR_AMD16_PERFEVTSEL3           0xC0010003
+#define MSR_AMD16_PMC0                  0xC0010004
+#define MSR_AMD16_PMC1                  0xC0010005
+#define MSR_AMD16_PMC2                  0xC0010006
+#define MSR_AMD16_PMC3                  0xC0010007
+
+#define MSR_AMD16_NB_PERFEVTSEL0        0xC0010240
+#define MSR_AMD16_NB_PERFEVTSEL1        0xC0010242
+#define MSR_AMD16_NB_PERFEVTSEL2        0xC0010244
+#define MSR_AMD16_NB_PERFEVTSEL3        0xC0010246
+#define MSR_AMD16_NB_PMC0               0xC0010241
+#define MSR_AMD16_NB_PMC1               0xC0010243
+#define MSR_AMD16_NB_PMC2               0xC0010245
+#define MSR_AMD16_NB_PMC3               0xC0010247
+
 #endif /* REGISTERS_H */
 
