@@ -5,13 +5,13 @@
  *
  *      Description:  Global  Types file
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
 #include <accessClient_types.h>
 #include <pci_types.h>
 #include <power_types.h>
+#include <thermal_types.h>
 #include <strUtil_types.h>
 #include <test_types.h>
 #include <barrier_types.h>
@@ -53,8 +54,13 @@
 #include <libperfctr_types.h>
 #include <multiplex_types.h>
 #include <numa_types.h>
+#include <pci_types.h>
 
 
+typedef struct {
+    uint64_t mask[2];
+} BitMask;
+
 /* #####   EXPORTED MACROS   ############################################## */
 
 #ifndef MIN
@@ -64,8 +70,10 @@
 #define MAX(x,y) ((x)>(y)?(x):(y))
 #endif
 
+#ifndef GLIB_MAJOR_VERSION
 #define TRUE  1
 #define FALSE 0
+#endif
 
 #define HLINE "-------------------------------------------------------------\n"
 #define SLINE "*************************************************************\n"
