@@ -5,13 +5,13 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -53,15 +53,31 @@ void
 power_init(int cpuId)
 {
     uint64_t flags;
-    uint32_t tmp;
     int hasRAPL = 0;
+    uint32_t info_register = 0x0;
 
     /* determine Turbo Mode features */
     double busSpeed;
 
     if ((cpuid_info.model == SANDYBRIDGE_EP) ||
             (cpuid_info.model == SANDYBRIDGE) ||
+            (cpuid_info.model == HASWELL) ||
+            (cpuid_info.model == HASWELL_EX) ||
+            (cpuid_info.model == IVYBRIDGE_EP) ||
             (cpuid_info.model == IVYBRIDGE))
+    {
+        hasRAPL = 1;
+        info_register = MSR_PKG_POWER_INFO;
+    }
+    else if (cpuid_info.model == ATOM_SILVERMONT_C)
+    {
+        hasRAPL = 1;
+        info_register = MSR_PKG_POWER_INFO_SILVERMONT;
+    }
+    else if ((cpuid_info.model == ATOM_SILVERMONT_E) ||
+             (cpuid_info.model == ATOM_SILVERMONT_F1) ||
+             (cpuid_info.model == ATOM_SILVERMONT_F2) ||
+             (cpuid_info.model == ATOM_SILVERMONT_F3))
     {
         hasRAPL = 1;
     }
@@ -89,13 +105,13 @@ power_init(int cpuId)
 
         for (int i=0; i < power_info.turbo.numSteps; i++)
         {
-            if ( i < 4 )
+            if (i < 8)
             {
-                power_info.turbo.steps[i] = busSpeed * (double) extractBitField(flags,8,i*8);
+                power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
             }
             else
             {
-                power_info.turbo.steps[i] = busSpeed * (double) extractBitField(flags>>(32),8,i*8);
+                power_info.turbo.steps[i] = power_info.turbo.steps[7];
             }
         }
     }
@@ -113,15 +129,40 @@ power_init(int cpuId)
         power_info.energyUnit = pow(0.5,(double) extractBitField(flags,5,8));
         power_info.timeUnit = pow(0.5,(double) extractBitField(flags,4,16));
 
-        flags = msr_read(cpuId, MSR_PKG_POWER_INFO);
-        power_info.tdp = (double) extractBitField(flags,15,0) * power_info.powerUnit;
-        power_info.minPower =  (double) extractBitField(flags,15,16) * power_info.powerUnit;
-        power_info.maxPower = (double) extractBitField(flags,15,32) * power_info.powerUnit;
-        power_info.maxTimeWindow = (double) extractBitField(flags,15,48) * power_info.timeUnit;
+        if (info_register != 0x0)
+        {
+            flags = msr_read(cpuId, info_register);
+            power_info.tdp = (double) extractBitField(flags,15,0) * power_info.powerUnit;
+            if (cpuid_info.model != ATOM_SILVERMONT_C)
+            {
+                power_info.minPower =  (double) extractBitField(flags,15,16) * power_info.powerUnit;
+                power_info.maxPower = (double) extractBitField(flags,15,32) * power_info.powerUnit;
+                power_info.maxTimeWindow = (double) extractBitField(flags,7,48) * power_info.timeUnit;
+            }
+            else
+            {
+                power_info.minPower = 0.0;
+                power_info.maxPower = 0.0;
+                power_info.maxTimeWindow = 0.0;
+            }
+        }
+        else
+        {
+            power_info.tdp = 0;
+            power_info.minPower = 0.0;
+            power_info.maxPower = 0.0;
+            power_info.maxTimeWindow = 0.0;
+        }
     }
     else
     {
         power_info.powerUnit = 0.0;
+        power_info.energyUnit = 0.0;
+        power_info.timeUnit = 0.0;
+        power_info.tdp = 0;
+        power_info.minPower = 0.0;
+        power_info.maxPower = 0.0;
+        power_info.maxTimeWindow = 0.0;
     }
 }
 
