@@ -5,13 +5,13 @@
  *
  *      Description:  Utility routines for strings. Depends on bstring lib.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -40,6 +40,7 @@
 #include <strUtil.h>
 #include <affinity.h>
 #include <cpuid.h>
+#include <pci.h>
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 static int
@@ -63,7 +64,7 @@ cpu_count(cpu_set_t* set)
         {
             if (l&(1UL<<i))
             {
-                s++;
+            s++;
             }
         }
     }
@@ -72,23 +73,21 @@ cpu_count(cpu_set_t* set)
 }
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
-int
-str2int(const char* str)
+int str2int(const char* str)
 {
     char* endptr;
     errno = 0;
     unsigned long val;
     val = strtoul(str, &endptr, 10);
-
     if ((errno == ERANGE && val == LONG_MAX )
-            || (errno != 0 && val == 0)) 
+        || (errno != 0 && val == 0))
     {
         ERROR;
     }
 
-    if (endptr == str) 
+    if (endptr == str)
     {
-        ERROR_PLAIN_PRINT(No digits were found);
+        ERROR_PRINT(Cannot parse string %s to digits, str);
     }
 
     return (int) val;
@@ -110,24 +109,19 @@ bstr_to_cpuset_physical(uint32_t* threads,  const_bstring q)
     {
         subtokens = bsplit(tokens->entry[i],'-');
 
-        if (numThreads > MAX_NUM_THREADS)
-        {
-            ERROR_PRINT(Number Of threads %d too large, numThreads);
-        }
-
         if( subtokens->qty == 1 )
         {
-            threads[numThreads] = str2int((char *) subtokens->entry[0]->data);
+            threads[numThreads] = str2int((char *) bdata(subtokens->entry[0]));
             numThreads++;
         }
         else if ( subtokens->qty == 2 )
         {
-            rangeBegin = str2int((char*) subtokens->entry[0]->data);
-            rangeEnd = str2int((char*) subtokens->entry[1]->data);
+            rangeBegin = str2int((char*) bdata(subtokens->entry[0]));
+            rangeEnd = str2int((char*) bdata(subtokens->entry[1]));
 
             if (!(rangeBegin <= rangeEnd))
             {
-                ERROR_PRINT(Range End %d bigger than begin %d,rangeEnd,rangeBegin);
+                ERROR_PRINT(Range End %d bigger than begin %d, rangeEnd, rangeBegin);
             }
 
             while (rangeBegin <= rangeEnd) {
@@ -142,6 +136,10 @@ bstr_to_cpuset_physical(uint32_t* threads,  const_bstring q)
         }
         bstrListDestroy(subtokens);
     }
+    if (numThreads > MAX_NUM_THREADS)
+    {
+        ERROR_PRINT(Number Of threads %d too large, numThreads);
+    }
 
     bstrListDestroy(tokens);
 
@@ -153,6 +151,7 @@ bstr_to_cpuset_logical(uint32_t* threads,  const_bstring q)
 {
     int i;
     uint32_t j;
+    int id;
     uint32_t tmpThreads[MAX_NUM_THREADS];
     int globalNumThreads=0;
     uint32_t numThreads=0;
@@ -178,10 +177,12 @@ bstr_to_cpuset_logical(uint32_t* threads,  const_bstring q)
             numThreads = bstr_to_cpuset_physical(tmpThreads, subtokens->entry[1]);
 
             for (j=0; j<numThreads; j++)
-            {
+                {
                 if (! (tmpThreads[j] >= domain->numberOfProcessors))
                 {
-                    threads[globalNumThreads++] = domain->processorList[tmpThreads[j]];
+                    id = (tmpThreads[j]/domain->numberOfCores) +
+                        (tmpThreads[j]%domain->numberOfCores) * cpuid_topology.numThreadsPerCore;
+                    threads[globalNumThreads++] = domain->processorList[id];
                 }
                 else
                 {
@@ -201,6 +202,165 @@ bstr_to_cpuset_logical(uint32_t* threads,  const_bstring q)
     return globalNumThreads;
 }
 
+#define PRINT_EXPR_ERR printf("SYNTAX ERROR: Expression must have the format E:<thread domain>:<num threads>[:chunk size>:<stride>]\n")
+
+uint32_t
+bstr_to_cpuset_expression(uint32_t* threads,  const_bstring qi)
+{
+    int i;
+    uint32_t j;
+    bstring q = (bstring) qi;
+    int globalNumThreads=0;
+    uint32_t numThreads=0;
+    struct bstrList* tokens;
+    struct bstrList* subtokens;
+    const AffinityDomain* domain;
+
+    bdelete (q, 0, 2);
+    tokens = bsplit(q,'@');
+
+    for (i=0;i<tokens->qty;i++)
+    {
+        subtokens = bsplit(tokens->entry[i],':');
+
+        if ( subtokens->qty == 2 )
+        {
+            domain =  affinity_getDomain(subtokens->entry[0]);
+
+            if (!domain)
+            {
+                ERROR_PRINT(Unknown domain ##%s##,bdata(subtokens->entry[0]));
+            }
+
+            numThreads = str2int(bdata(subtokens->entry[1]));
+
+            if (numThreads > domain->numberOfProcessors)
+            {
+                ERROR_PRINT(Invalid processor id requested. Avaialable 0-%d,
+                            domain->numberOfProcessors-1);
+            }
+
+            for (j=0; j<numThreads; j++)
+            {
+                threads[globalNumThreads++] = domain->processorList[j];
+            }
+        }
+        else if ( subtokens->qty == 4 )
+        {
+            int counter;
+            int currentId = 0;
+            int startId = 0;
+            int chunksize =  str2int(bdata(subtokens->entry[2]));
+            int stride =  str2int(bdata(subtokens->entry[3]));
+            domain = affinity_getDomain(subtokens->entry[0]);
+
+            if (!domain)
+            {
+                ERROR_PRINT(Unknown domain ##%s##,bdata(subtokens->entry[0]));
+            }
+
+            numThreads = str2int(bdata(subtokens->entry[1]));
+
+            if (numThreads > domain->numberOfProcessors)
+            {
+                ERROR_PRINT(Invalid number of processors requested. Available 0-%d,
+                            domain->numberOfProcessors-1);
+            }
+
+
+            counter = 0;
+            for (j=0; j<numThreads; j+=chunksize)
+            {
+                for(i=0;i<chunksize && j+i<numThreads ;i++)
+                {
+                    threads[globalNumThreads++] = domain->processorList[counter+i];
+                }
+                counter += stride;
+                if (counter >= domain->numberOfProcessors)
+                {
+                    counter = 0;
+                }
+            }
+        }
+        else
+        {
+            PRINT_EXPR_ERR;
+            ERROR_PLAIN_PRINT(Parse Error);
+        }
+        bstrListDestroy(subtokens);
+    }
+
+    bstrListDestroy(tokens);
+
+    return globalNumThreads;
+}
+
+uint32_t
+bstr_to_cpuset_scatter(uint32_t* threads,  const_bstring qi)
+{
+    int domainId = 0;
+    int id = 0;
+    int threadId = 0;
+    bstring q = (bstring) qi;
+    bstring domaintag;
+    int globalNumThreads=0;
+    struct bstrList* subtokens;
+    int numberOfDomains = 0;
+    AffinityDomain* domain;
+    AffinityDomain* tmpDomainPtr;
+
+    domain = (AffinityDomain*) malloc(cpuid_topology.numHWThreads * sizeof(AffinityDomain));
+
+    subtokens = bsplit(q,':');
+
+    if ( subtokens->qty == 2 )
+    {
+        for(int i =0;;i++)
+        {
+            domaintag = bformat("%s%d",bdata(subtokens->entry[0]),i);
+            tmpDomainPtr = (AffinityDomain*) affinity_getDomain(domaintag);
+
+            if (tmpDomainPtr == NULL)
+            {
+                break;
+            }
+            else
+            {
+                memcpy(domain+i,tmpDomainPtr,sizeof(AffinityDomain));
+                numberOfDomains++;
+            }
+        }
+
+        threads[globalNumThreads++] = domain[domainId].processorList[0];
+
+        for (uint32_t i=1; i<cpuid_topology.numHWThreads; i++)
+        {
+            domainId = i%numberOfDomains;
+
+            if (domainId == 0)
+            {
+                threadId++;
+            }
+
+            id = (threadId/domain->numberOfCores) +
+                (threadId%domain->numberOfCores) * cpuid_topology.numThreadsPerCore;
+
+            threads[globalNumThreads++] = domain[domainId].processorList[id];
+        }
+    }
+    else
+    {
+        PRINT_EXPR_ERR;
+        ERROR_PLAIN_PRINT(Parse Error);
+    }
+    bstrListDestroy(subtokens);
+    free(domain);
+
+    return globalNumThreads;
+}
+
+
+
 #define CPUSET_ERROR  \
     if (cpuid_isInCpuset()) {  \
         ERROR_PLAIN_PRINT(You are running inside a cpuset. In cpusets only logical pinning inside set is allowed!);  \
@@ -217,21 +377,33 @@ bstr_to_cpuset(int* threadsIN,  const_bstring q)
     cpu_set_t cpu_set;
     uint32_t numThreads;
     bstring domainStr = bformat("NSCM");
+    const_bstring  scatter = bformat("scatter");
     struct bstrList* tokens;
     CPU_ZERO(&cpu_set);
     sched_getaffinity(0,sizeof(cpu_set_t), &cpu_set);
     uint32_t* threads = (uint32_t*) threadsIN;
 
-
     if (binchr (q, 0, domainStr) !=  BSTR_ERR)
     {
         CPUSET_ERROR;
-        numThreads =  bstr_to_cpuset_logical(threads,q);
+
+        if (binstr (q, 0 , scatter ) !=  BSTR_ERR)
+        {
+          numThreads =  bstr_to_cpuset_scatter(threads,q);
+        }
+        else if (bstrchr (q, 'E') !=  BSTR_ERR)
+        {
+          numThreads =  bstr_to_cpuset_expression(threads,q);
+        }
+        else
+        {
+          numThreads =  bstr_to_cpuset_logical(threads,q);
+        }
     }
     else if (bstrchr (q, 'L') !=  BSTR_ERR)
     {
         uint32_t count = cpu_count(&cpu_set);
-        printf("Using logical numbering within cpuset %d\n",count);
+
         tokens = bsplit(q,':');
         numThreads = bstr_to_cpuset_physical(threads,tokens->entry[1]);
 
@@ -245,13 +417,19 @@ bstr_to_cpuset(int* threadsIN,  const_bstring q)
 
         for (i=0; i < numThreads; i++)
         {
-            if (!(threads[i] > count))
+            if (!(threads[i] >= count))
             {
                 threads[i] = cpuMapping[threads[i]];
             }
             else
             {
-                ERROR_PRINT(Request cpu out of range of max %d,count);
+                fprintf(stderr, "Available CPUs: ");
+                for (int j=0; j< num-1;j++)
+                {
+                    fprintf(stderr, "%d,", cpuMapping[j]);
+                }
+                fprintf(stderr, "%d\n", cpuMapping[num-1]);
+                ERROR_PRINT(Index %d out of range.,threads[i]);
             }
         }
         bstrListDestroy(tokens);
@@ -277,7 +455,7 @@ bstr_to_eventset(StrUtilEventSet* set, const_bstring q)
     tokens = bsplit(q,',');
     set->numberOfEvents = tokens->qty;
     set->events = (StrUtilEvent*)
-        malloc(set->numberOfEvents * sizeof(StrUtilEvent));
+    malloc(set->numberOfEvents * sizeof(StrUtilEvent));
 
     for (i=0;i<tokens->qty;i++)
     {
@@ -285,7 +463,14 @@ bstr_to_eventset(StrUtilEventSet* set, const_bstring q)
 
         if ( subtokens->qty != 2 )
         {
-            ERROR_PLAIN_PRINT(Error in parsing event string);
+          
+            fprintf(stderr, "Cannot parse event string %s, probably missing counter name\n"
+                          ,bdata(tokens->entry[i]));
+            fprintf(stderr, "Format: <eventName>:<counter>,...\n");
+            msr_finalize();
+            pci_finalize();
+            exit(EXIT_FAILURE);
+
         }
         else
         {
@@ -299,48 +484,44 @@ bstr_to_eventset(StrUtilEventSet* set, const_bstring q)
     bstrListDestroy(tokens);
 }
 
-FILE* 
+FILE*
 bstr_to_outstream(const_bstring argString, bstring filter)
 {
     int i;
     char* cstr;
     FILE* STREAM;
-    int numberOfFields;
     struct bstrList* tokens;
     bstring base;
     bstring suffix = bfromcstr(".");
     bstring filename;
 
     /* configure filter */
+    tokens = bsplit(argString,'.');
+
+    if (tokens->qty < 2)
     {
-        tokens = bsplit(argString,'.');
-
-        if (tokens->qty < 2)
-        {
-            fprintf(stderr, "Outputfile has no filetype suffix!\n");
-            fprintf(stderr, "Add suffix .txt for raw output or any supported filter suffix.\n");
-            exit(EXIT_FAILURE);
-        }
-
-        base = bstrcpy(tokens->entry[0]);
-
-        if (biseqcstr(tokens->entry[1],"txt"))
-        {
-            bassigncstr(filter, "NO");
-        }
-        else
-        {
-            bassigncstr(filter, TOSTRING(LIKWIDFILTERPATH));
-            bconchar(filter,'/');
-            bconcat(filter,tokens->entry[1]);
-        }
-
-        bconcat(suffix,tokens->entry[1]);
-        bstrListDestroy(tokens);
+        fprintf(stderr, "Outputfile has no filetype suffix!\n");
+        fprintf(stderr, "Add suffix .txt for raw output or any supported filter suffix.\n");
+        exit(EXIT_FAILURE);
     }
 
+    base = bstrcpy(tokens->entry[0]);
+
+    if (biseqcstr(tokens->entry[1],"txt"))
+    {
+        bassigncstr(filter, "NO");
+    }
+    else
+    {
+        bassigncstr(filter, TOSTRING(LIKWIDFILTERPATH));
+        bconchar(filter,'/');
+        bconcat(filter,tokens->entry[1]);
+    }
+
+    bconcat(suffix,tokens->entry[1]);
+    bstrListDestroy(tokens);
+
     tokens = bsplit(base,'_');
-    numberOfFields = tokens->qty;
 
     if (tokens->qty < 1)
     {
@@ -414,7 +595,8 @@ bstr_to_outstream(const_bstring argString, bstring filter)
 }
 
 
-uint64_t bstr_to_doubleSize(const_bstring str, DataType type)
+uint64_t
+bstr_to_doubleSize(const_bstring str, DataType type)
 {
     bstring unit = bmidstr(str, blength(str)-2, 2);
     bstring sizeStr = bmidstr(str, 0, blength(str)-2);
@@ -425,11 +607,13 @@ uint64_t bstr_to_doubleSize(const_bstring str, DataType type)
     switch (type)
     {
         case SINGLE:
-            bytesize = 4;
+        case SINGLE_RAND:
+            bytesize = sizeof(float);
             break;
 
         case DOUBLE:
-            bytesize = 8;
+        case DOUBLE_RAND:
+            bytesize = sizeof(double);
             break;
     }
 
@@ -444,7 +628,8 @@ uint64_t bstr_to_doubleSize(const_bstring str, DataType type)
     return junk;
 }
 
-void bstr_to_interval(const_bstring str, struct timespec* interval)
+void
+bstr_to_interval(const_bstring str, struct timespec* interval)
 {
     int size;
     int pos;
@@ -454,7 +639,9 @@ void bstr_to_interval(const_bstring str, struct timespec* interval)
     {
         if (pos != (blength(str)-1))
         {
-            ERROR_PLAIN_PRINT(Parsing of daemon interval failed);
+            fprintf(stderr, "You need to specify a time unit s or ms like 200ms\n");
+            msr_finalize();
+            exit(EXIT_FAILURE);
         }
 
         /* unit is ms */
@@ -484,15 +671,18 @@ void bstr_to_interval(const_bstring str, struct timespec* interval)
     }
     else
     {
-        ERROR_PLAIN_PRINT(Parsing of daemon interval failed);
+        fprintf(stderr, "You need to specify a time unit s or ms like 200ms\n");
+        msr_finalize();
+        exit(EXIT_FAILURE);
     }
 }
 
 
-void bstr_to_workgroup(Workgroup* group,
-        const_bstring str,
-        DataType type,
-        int numberOfStreams)
+void
+bstr_to_workgroup(Workgroup* group,
+    const_bstring str,
+    DataType type,
+    int numberOfStreams)
 {
     uint32_t i;
     int parseStreams = 0;
@@ -510,7 +700,7 @@ void bstr_to_workgroup(Workgroup* group,
         threadInfo = bstrcpy(tokens->entry[0]);
         streams = bstrcpy(tokens->entry[1]);
         parseStreams = 1;
-    } 
+    }
     else if (tokens->qty == 1)
     {
         threadInfo = bstrcpy(tokens->entry[0]);
@@ -523,14 +713,65 @@ void bstr_to_workgroup(Workgroup* group,
     bstrListDestroy (tokens);
     tokens = bsplit(threadInfo,':');
 
-    if (tokens->qty == 3)
+    if (tokens->qty == 5)
+    {
+        uint32_t maxNumThreads;
+        int chunksize;
+        int stride;
+        int counter;
+        int currentId = 0;
+        int startId = 0;
+
+        domain = affinity_getDomain(tokens->entry[0]);
+
+        if (domain == NULL)
+        {
+          fprintf(stderr, "Error: Domain %s not available on current machine.\nTry likwid-bench -p for supported domains.",
+              bdata(tokens->entry[0]));
+          exit(EXIT_FAILURE);
+        }
+
+        group->size = bstr_to_doubleSize(tokens->entry[1], type);
+        group->numberOfThreads = str2int(bdata(tokens->entry[2]));
+        chunksize = str2int(bdata(tokens->entry[3]));
+        stride = str2int(bdata(tokens->entry[4]));
+        maxNumThreads = (domain->numberOfProcessors / stride) * chunksize;
+
+        if (group->numberOfThreads > maxNumThreads)
+        {
+          fprintf(stderr, "Error: Domain %s supports only up to %d threads with used expression.\n",
+                        bdata(tokens->entry[0]), maxNumThreads);
+          exit(EXIT_FAILURE);
+        }
+
+        group->processorIds = (int*) malloc(group->numberOfThreads * sizeof(int));
+
+        counter = chunksize;
+
+        for (i=0; i<group->numberOfThreads; i++)
+        {
+            if (counter)
+            {
+                group->processorIds[i] = domain->processorList[currentId++];
+            }
+            else
+            {
+                startId += stride;
+                currentId = startId;
+                group->processorIds[i] = domain->processorList[currentId++];
+                counter = chunksize;
+            }
+            counter--;
+        }
+    }
+    else if (tokens->qty == 3)
     {
         domain = affinity_getDomain(tokens->entry[0]);
 
         if (domain == NULL)
         {
-            fprintf(stderr, "Error: Domain %s not available on current machine.\nTry likwid-bench -p for supported domains.",
-                    bdata(tokens->entry[0]));
+            fprintf(stderr, "Error: Domain %s not available on current machine.\n", bdata(tokens->entry[0]));
+            fprintf(stderr, "Try likwid-bench -p for supported domains.\n");
             exit(EXIT_FAILURE);
         }
 
@@ -540,7 +781,7 @@ void bstr_to_workgroup(Workgroup* group,
         if (group->numberOfThreads > domain->numberOfProcessors)
         {
             fprintf(stderr, "Error: Domain %s supports only up to %d threads.\n",
-                    bdata(tokens->entry[0]),domain->numberOfProcessors);
+                            bdata(tokens->entry[0]),domain->numberOfProcessors);
             exit(EXIT_FAILURE);
         }
 
@@ -550,13 +791,21 @@ void bstr_to_workgroup(Workgroup* group,
         {
             group->processorIds[i] = domain->processorList[i];
         }
-    } 
+    }
     else if (tokens->qty == 2)
     {
         domain = affinity_getDomain(tokens->entry[0]);
+
+        if (domain == NULL)
+        {
+            fprintf(stderr, "Error: Domain %s not available on current machine.\n",
+                            bdata(tokens->entry[0]));
+            fprintf(stderr, "Try likwid-bench -p for supported domains.\n");
+            exit(EXIT_FAILURE);
+        }
+
         group->size = bstr_to_doubleSize(tokens->entry[1], type);
         group->numberOfThreads = domain->numberOfProcessors;
-
         group->processorIds = (int*) malloc(group->numberOfThreads * sizeof(int));
 
         for (i=0; i<group->numberOfThreads; i++)
@@ -566,7 +815,7 @@ void bstr_to_workgroup(Workgroup* group,
     }
     else
     {
-        ERROR_PLAIN_PRINT(Error in parsing workgroup string);
+    ERROR_PLAIN_PRINT(Error in parsing workgroup string);
     }
 
     bstrListDestroy(tokens);
@@ -589,20 +838,20 @@ void bstr_to_workgroup(Workgroup* group,
 
             if ( subtokens->qty == 3 )
             {
-               int index = str2int(bdata(subtokens->entry[0]));
-               if (index >= numberOfStreams)
-               {
-                   ERROR_PRINT(Stream Index %d out of range,index);
-               }
-               group->streams[index].domain = bstrcpy(subtokens->entry[1]);
-               group->streams[index].offset = str2int(bdata(subtokens->entry[2]));
+                int index = str2int(bdata(subtokens->entry[0]));
+                if (index >= numberOfStreams)
+                {
+                    ERROR_PRINT(Stream Index %d out of range,index);
+                }
+                group->streams[index].domain = bstrcpy(subtokens->entry[1]);
+                group->streams[index].offset = str2int(bdata(subtokens->entry[2]));
             }
             else if ( subtokens->qty == 2 )
             {
                 int index = str2int(bdata(subtokens->entry[0]));
                 if (index >= numberOfStreams)
                 {
-                   ERROR_PRINT(Stream Index %d out of range,index);
+                    ERROR_PRINT(Stream Index %d out of range,index);
                 }
                 group->streams[index].domain = bstrcpy(subtokens->entry[1]);
                 group->streams[index].offset = 0;
@@ -668,12 +917,12 @@ bSecureInput (int maxlen, char* vgcCtx) {
         if (i+1 >= b->mlen) {
 
             /* Double size, but deal with unusual case of numeric
-               overflows */
+             overflows */
 
-            if ((m = b->mlen << 1)   <= b->mlen     &&
-                    (m = b->mlen + 1024) <= b->mlen &&
-                    (m = b->mlen + 16)   <= b->mlen &&
-                    (m = b->mlen + 1)    <= b->mlen)
+            if ((m = b->mlen << 1)   <= b->mlen &&
+                (m = b->mlen + 1024) <= b->mlen &&
+                (m = b->mlen + 16)   <= b->mlen &&
+                (m = b->mlen + 1)    <= b->mlen)
             {
                 t = NULL;
             }
@@ -705,7 +954,8 @@ bSecureInput (int maxlen, char* vgcCtx) {
 }
 
 
-int bJustifyCenter (bstring b, int width) 
+int
+bJustifyCenter (bstring b, int width) 
 {
     unsigned char space  = ' ';
     int alignSpace = (width - b->slen) / 2;
