@@ -5,13 +5,13 @@
  *
  *      Description:  Module implementing a tree data structure
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   3.1.3
+ *      Released:  4.11.2014
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2014 Jan Treibig
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -48,17 +48,32 @@ tree_init(TreeNode** root, int id)
 void
 tree_print(TreeNode* nodePtr)
 {
-    if (nodePtr != NULL) 
-    {
-        printf("%d ", nodePtr->id);
-        TreeNode* walker;    
-        walker = nodePtr->llink;
+    int level = 0;
 
-        while (walker != NULL)
+    if (nodePtr != NULL)
+    {
+
+        TreeNode* digger;
+        TreeNode* walker;
+
+        digger = nodePtr->llink;
+
+        while (digger != NULL)
         {
-            tree_print(walker);
+            printf("\n Level %d:\n", level++);
+            printf("%d ", digger->id);
+            walker = digger->rlink;
+
+            while (walker != NULL)
+            {
+            printf("%d ", walker->id);
             walker = walker->rlink;
+            }
+
+            digger = digger->llink;
         }
+
+        printf("\n ");
     }
 }
 
